//
//    mar'04
//    jan'05 - version v02a
// -------------------------------------------------------------------------------------------------
//                                                                       the FLEUR project
//                                                                       author: anton.empl@cern.ch
// -- FLUKA user back-end project
//    see header file (FbeHist.h) for comments
//
//


#include "FbeColors.h"
#include "FbeHist.h"

#include "aMessageBox.h"

ClassImp(FbeHist)



//
// --- one time setup of our FLUKA printer optimized colors to be given to the color palette
//     this to clear/initializes up to 99 color cells for later use
//    ------------------------------------------------------------------------------------------------

FbeHist::FbeHist(const char *n, const char *t, int nx,double xl,double xh, int ny,double yl,double yh)
        :TH2F(n,t,nx,xl,xh,ny,yl,yh)
{

if (!gFColor[0]) {
    for (int i=0;i<22;i++) { aFC[i][0]= bFC[i][0]; aFC[i][1]= bFC[i][1]; aFC[i][2]= bFC[i][2]; }
    for (int i=22;i<100;i++) aFC[i][0]= aFC[i][1]= aFC[i][2]= 1.;
    for (int i=0;i<100;i++) { gFColor[i]= new TColor(1106+i, .0,.0,.0 ,""); gFPalette[i]= 1106+i; }
    }

_swap= 0;

}



//
// --- our first task here is to have a simple standard sorta draw() with the few options
//     we want to implement and (semi automatic) nice font sizes
//     important feature here is how to determine the actual colors for the palette
//     several different approaches: (default)floating/anchored/(not anymore)fixed
//     mode permits to turn off drawing the palette (and select a different one with <0)
//
//     we use -1/cpd for fractional cases
//     need more checking but works to first order
//    ------------------------------------------------------------------------------------------------
//
//     problem: when only a single decade then root doesn put any numbers
//              rootfop/fixed were removed for now

//
// --- oct'06 24-07:51
//     bug for cpd>100 - ie we tried to be too simple in having only 100 root color cells
//     also found that fractional feature seemed to be turned off (on purpose?)
//     push preparePalette and cpd to double


double
FbeHist::preparePalette(double anchored, double colorsPerDecade, int mode, double scale, double rooftop)
{
double cpd=1;
int cpdInt=1;
double d,u, hi=_max*scale, lo=_min*scale;
int i,j, twenty2=22;

//
// --- trouble with single bin, hence hi==lo and we used to bomb (jan'05 22-07:14)
//     then decide on just less than 200 color cells, the best resolution default

if (anchored>0) {
  if (hi-anchored<1e-99) { cpdInt= 1; cpd= 1; }
    else {
           if (fabs(colorsPerDecade)>.0001) { cpd= colorsPerDecade;
                                              cpdInt= (cpd>1) ? (int)cpd:(int)round(-1/cpd); }
             else { 
                    cpd= twenty2/ceil(log10(hi/anchored));
                    cpdInt= (cpd>1) ? (int)cpd:(int)round(-1/cpd);

                    int N= cpdInt;
                    if (N>0)  while ((int)ceil((log10(hi)-log10(lo))*N-.1e-3)>199) N-= 1;
                      else  while ((int)ceil((log10(hi)-log10(lo))/abs(N)-.1e-3)>199) N-= 1;
                    cpdInt= N;
                    cpd= (cpdInt>0) ? cpdInt:-1./cpdInt;
                  }
         }}

  else {
         if (hi-lo<1e-99) { cpdInt= 1; cpd= 1; }
           else { cpd= twenty2/ceil(log10(hi/lo));
                  cpdInt= (cpd>1) ? (int)cpd:(int)round(-1/cpd);
                  int N= cpdInt;
                  if (N>0)  while ((int)ceil((log10(hi)-log10(lo))*N-.1e-3)>199) N-= 1;
                    else  while ((int)ceil((log10(hi)-log10(lo))/abs(N)-.1e-3)>199) N-= 1;
                  cpdInt= N;
                  cpd= (cpdInt>0) ? cpdInt:-1./cpdInt;

           // -user requested-
                  if (fabs(colorsPerDecade)>.0001) {
                    int uN= (colorsPerDecade>1) ? (int)colorsPerDecade:(int)round(-1/colorsPerDecade);
                    double cpdU= (uN>0) ? uN:-1./uN;
                    // if (uN<=N) { cpd= cpdU; cpdInt= uN; }
                    cpd= cpdU; cpdInt= uN;
                    }
                }}

DEBUG(printf(" ===-===> mode %i -- hi %g lo %g\n",mode,log10(hi),log10(lo));)
DEBUG(printf(" ===-===> anchored %g -- N in %g  --  cpd %g (int)cpd %i\n", anchored,colorsPerDecade,cpd,cpdInt);)
colorsPerDecade= cpd;
DEBUG(printf(" ===-===> cpdInt %i colorsPerDecade %g\n",cpdInt,colorsPerDecade);)

if (mode<2) gStyle->SetPalette(1,0);

  else { double a,t;
         int nt,nd,na;
         double xx,dd=1/cpd,yy;

         t= u= niceColorEdge(hi,cpdInt,1);
         a= d= niceColorEdge(lo,cpdInt,0);

         if (anchored>0) { a= niceColorEdge(anchored,cpdInt,0);
                           t= niceColorEdge(pow(10.,log10(a)+twenty2/cpd),cpdInt,1);  }

         nd= (int)ceil((log10(u)-log10(d))*cpd-.1e-3);
         na= (int)ceil((log10(u)-log10(a))*cpd-.1e-3);
         nt= twenty2;

         DEBUG(printf(" ---du---> %3i %13.8g %13.9g - %9.5f %9.5f\n",nd,d,u,log10(d),log10(u));)
         DEBUG(printf(" ---at---> %3i %13.8g %13.9g - %9.5f %9.5f\n",nt,a,t,log10(a),log10(t));)

         int i2j= nd-1;
         if (anchored>0) { i2j= twenty2-1;
                           d= a; nd= na;
                           if (nd>twenty2) { u= t; nd= nt; }}

         if (nd>100) { printf(" -warning- using > 100 colors, might not work\n"); nd= 100; }

         for (i=0;i<nd;i++) { j= MIN(i2j-i,23);
                              if (j<0) { printf(" i=%3i j<0 %4i\n",i,j); j= 0; }
                              xx= log10(u)-dd*i; yy= pow(10.,xx);
                              DEBUG(if (i==0||nd-i==1) printf(" -c- %3i %3i  %7.3f %19.16f\n",i,j,xx,yy);)
                              gFColor[i]->SetRGB(aFC[j][0],aFC[j][1],aFC[j][2]); }
         gStyle->SetPalette(nd,gFPalette);

         DEBUG(printf(" ===-===> min %g max %g N %i\n",d,u,nd);)

         SetMinimum(d); SetMaximum(u); SetContour(nd);
       }

return (colorsPerDecade<1) ? .001*(int)round(colorsPerDecade*1000):colorsPerDecade;
}



//
// --- this helper finds lower and upper edges such that we show our palette with an integral
//     number of color fields
//    ------------------------------------------------------------------------------------------------

double FbeHist::niceColorEdge(double value, int colorsPerDecade, int updown)
{

double val=value,fraction,log10of, zero=1e-31, a,b,newValue;
if (fabs(val)<zero) return (val);

val*= (updown) ? .99999 : 1.00001;
fraction= (colorsPerDecade>0) ? 1/(double)colorsPerDecade:(double)colorsPerDecade*-1;
log10of= log10(val);

a= log10of/fraction;
b= (updown) ? ceil(a):floor(a);
newValue= pow(10.,b*fraction);

return (newValue);

}

