//
//    mar'04
//    jan'05 - version v02a
// -------------------------------------------------------------------------------------------------
//                                                                       the FLEUR project
//                                                                       author: anton.empl@cern.ch
// -- FLUKA user back-end project
//    lets try and build our modified/specific histogram class on top of TH2F
//    have a few select constructors at first and a way to set some attributes for the palette
//    taken all built-in variables back to regular C/C++ types
//
//
//    13'mar  we now have a FLUKA type 'anchored' palette available if I undestand the philosophy
//            behind it ok - pick a low anchor, count up the 22 time colorsPerDecade, that's the
//            full and fixed palette range, then only draw from 'edged' _min to _max
//            select with anchored>0
//


#ifndef __FBEHIST_H
#define __FBEHIST_H

#include <stdlib.h>

#define DEBUG(arg)      {if (getenv("DEBUG")) arg;}

//
// --- a couple little helpful macros
//

#undef MIN
#define MIN(x,y) ((x) < (y) ? (x) : (y))
#undef MAX
#define MAX(x,y) ((x) > (y) ? (x) : (y))



#include <TROOT.h>
#include <TObject.h>
#include <TH1.h>
#include <TH2.h>
#include <TH2F.h>
#include <TStyle.h>
#include <TGMsgBox.h>

#include <math.h>


class FbeHist : public TH2F
{

private:

double _min;                    // these two we use since we manipulate the actual
double _max;                    // histogram values in context of our special palette
int   _swap;                    // our flag to facilitate swapping our histograms
int   _rphi;                    // indicate if this is a r/phi graph

public:

FbeHist() {};                                    // this one chains into TH2F
FbeHist(const char *n,const char *t, int nx, double xl, double xh, int ny, double yl, double yh);
virtual ~FbeHist() {};

// inline int preparePalette() { return preparePalette(0,0,1); }
double preparePalette(double anchored=0,double cpd=0,int mode=1,double scale=1, double rootftop=0);

inline void setRange(double x0,double x1,double y0,double y1) { GetXaxis()->SetRangeUser(x0,x1);
                                                                GetYaxis()->SetRangeUser(y0,y1); }
inline int *getRange() { int *tmp= new int[4];
                         tmp[0]= GetXaxis()->GetFirst()-1; tmp[1]= GetXaxis()->GetLast();
                         tmp[2]= GetYaxis()->GetFirst()-1; tmp[3]= GetYaxis()->GetLast();
                         return (tmp); }

inline void minimum(double a) { _min= a; }
inline void maximum(double a) { _max= a; }

inline void rphi(int a) { _rphi= a; }
inline int rphi() { return _rphi; }
inline int swap(int mode=0) { if (mode) _swap= (_swap)?0:1; return (_swap); }

inline double getmin() { return _min; }
inline double getmax() { return _max; }

private:
double niceColorEdge(double val, int boxesPerDecade, int updown);

ClassDef(FbeHist,1)

};


#endif
