
//
//    (feb'07 19-02:07) - version v.07a
//                                                                                 the FLEUR project
// -------------------------------------------------------------------------------------------------
//                                                                        author: anton.empl@cern.ch
// --- FLUKA user back-end project -Fbe-
//     this class implements our special binary reading from files (USRBIN only at this point)
//


#ifndef __MFBEIO_H
#define __MFBEIO_H


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

#define DEBUG(arg)      {if (getenv("DEBUG")) arg;}

#include <TObject.h>


#include "swab32.h"

#define NR_OF_BYTES_4_1ST_LINE_IN_FILE 120
#define CURRENT_USRBIN_FILE_TYPE 86

enum FBE_endian { FBE_littleEndian,FBE_bigEndian,FBE_somethingElse,FBE_ascii };
enum FBE_usrDataType { FBE_notDefined,FBE_usrbinCurrent,FBE_usryield,FBE_usrbdx,FBE_plotgeom };


class FbeIO 
{


protected:

int              _status;                 //
char             _filename[256];          //
int              _fd;                     //

FBE_endian       _endianess;              //
FBE_usrDataType  _usrDataType;            //


public:

FbeIO(const char *aName);
virtual ~FbeIO();


protected:

void identify(void);


inline int read(int fd, char* buf, int bytes) { return (::read(fd,buf,bytes)); }

inline int read(int fd, float* buf, int bytes) { int *ibuf= (int*)buf;
                                                 int n= read(fd,ibuf,bytes); return (n); }

inline int read(int fd, int* buf, int bytes) { int n= ::read(fd,buf,bytes);
                                               if (_endianess==FBE_bigEndian)
                                                 for (int i=0;i<bytes/4;i++) buf[i]= swab32(buf[i]);
                                               return (n); }


inline int read(int fd, char* buf) { int bytes, n= read(fd,&bytes,4); 
                                     n= read(fd,buf,bytes); n= read(fd,&bytes,4); return (bytes); }
inline int read(int fd, double* buf) { int bytes, n= read(fd,&bytes,4); int *ibuf= (int*)buf;
                                       n= read(fd,ibuf,bytes); n= read(fd,&bytes,4); return (bytes); }
inline int read(int fd, int* buf) { int bytes, n= read(fd,&bytes,4); 
                                    n= read(fd,buf,bytes); n= read(fd,&bytes,4); return (bytes); }

ClassDef(FbeIO,1)

}; // FbeIO - class related to reading for the -Fbe- project


#endif
