
//
//    mar'04
//    jan'05 - version v.03
//    (feb'07 19-02:07) - version v.07a
//                                                                                 the FLEUR project
// -------------------------------------------------------------------------------------------------
//                                                                                anton.empl@cern.ch
//                                                                   (Katey Covin, Matt LeBourgeois)
// --- FLUKA user back-end project -Fbe-
//     see header file (FbePaletteAxis.h) for comments
//
//     many changes occured for root, we should check if their new palette class is flexible
//     enough so that we can use it instead of our own here
//


#include "FbePaletteAxis.h"

ClassImp(FbePaletteAxis)



FbePaletteAxis::FbePaletteAxis(Double_t x1,Double_t y1,Double_t x2,Double_t y2,TH1 *h)
  :TPaletteAxis(x1,y1,x2,y2,h)
{
   fAxis.SetLabelOffset(-.09);
   fAxis.SetLabelSize(.035);

} // FbePaletteAxis


FbePaletteAxis::~FbePaletteAxis()
{
} // ~FbePaletteAxis



//    taken from root src tree
//    modifications are indicated by  -ae-
//______________________________________________________________________________
void FbePaletteAxis::Paint(Option_t *)
{

   ConvertNDCtoPad();

   SetFillStyle(1001);
   Double_t ymin = fY1;
   Double_t ymax = fY2;
   Double_t xmin = fX1;
   Double_t xmax = fX2;
   Double_t wmin = fH->GetMinimum();
   Double_t wmax = fH->GetMaximum();
   Double_t wlmin = wmin;
   Double_t wlmax = wmax;
   Double_t y1,y2,w1,w2,zc;
   if (gPad->GetLogz()) {
      if (wmin <= 0 && wmax > 0) wmin = 0.001*wmax;
      wlmin = TMath::Log10(wmin);
      wlmax = TMath::Log10(wmax);
   }
   Double_t ws    = wlmax-wlmin;
   Int_t ncolors = gStyle->GetNumberOfColors();
   Int_t ndivz = TMath::Abs(fH->GetContour());
   Int_t theColor,color;
   Double_t scale = ndivz/(wlmax - wlmin);
   for (Int_t i=0;i<ndivz;i++) {

      zc = fH->GetContourLevel(i);
      if (fH->TestBit(TH1::kUserContour) && gPad->GetLogz())
         zc = TMath::Log10(zc);
      w1 = zc;
      if (w1 < wlmin) w1 = wlmin;

      w2 = wlmax;
      if (i < ndivz-1) {
         zc = fH->GetContourLevel(i+1);
         if (fH->TestBit(TH1::kUserContour) && gPad->GetLogz())
            zc = TMath::Log10(zc);
         w2 = zc;
      }

      if (w2 <= wlmin) continue;
      y1 = ymin + (w1-wlmin)*(ymax-ymin)/ws;
      y2 = ymin + (w2-wlmin)*(ymax-ymin)/ws;

      if (fH->TestBit(TH1::kUserContour)) {
         // printf(" -debug-> FbePaletteAxis - kUserContour\n");
         color = i;
      } else {
         color = Int_t(0.01+(w1-wlmin)*scale);
      }

      theColor = Int_t((color+0.99)*Double_t(ncolors)/Double_t(ndivz));
      SetFillColor(gStyle->GetColorPalette(theColor));
      TAttFill::Modify();
//    gPad->PaintBox(xmin,y1,xmax,y2);
//   ------------------------------------------------- -ae- --- oct'03 -------------------------
      gPad->PaintBox(xmin+.3*(xmax-xmin),y1,xmax,y2);
   }

   Int_t ndiv  = fH->GetZaxis()->GetNdivisions()%100; //take primary divisions only
   char chopt[5] = "";
   chopt[0] = 0;

// ----------------------------- -ae- --- oct'03 -----------------------------------------------
// strcat(chopt, "+L");

   if (ndiv < 0) {
      ndiv =TMath::Abs(ndiv);
      strcat(chopt, "N");
   }

     //  printf(" -palette- ndiv %3i ws %f\n",ndiv,ws);

   if (gPad->GetLogz()) {
      wmin = TMath::Power(10.,wlmin);
      wmax = TMath::Power(10.,wlmax);
      strcat(chopt, "G");
   }
   fAxis.PaintAxis(xmax,ymin,xmax,ymax,wmin,wmax,ndiv,chopt);
}

