
//
//    mar'04
//    jan'05 - version v.03
//    (feb'07 19-02:07) - version v.07a
//                                                                                 the FLEUR project
// -------------------------------------------------------------------------------------------------
//                                                                                anton.empl@cern.ch
// --- FLUKA user back-end project -Fbe-
//     see header file (FbePlotgeom.h) for additional comments
//


#include "FbePlotgeom.h"


ClassImp(FbePlotgeom)

static unsigned char *_all,*_aLine;

int MXXMDF=710+1;

//
// --- our more flexible way of dealing with the cut through the FLUKA setup
//     here we use a wrapper around pgmdrv - the fortran FLUKA geometry driver routine
//     modified such that we are able to scan multiple times with only a single read through
//     the respective geometry information - this information is read from the FLUKA input
//     file - see libScan.a / scanGeometry.c
//    ------------------------------------------------------------------------------------------------
//

FbePlotgeom::FbePlotgeom(TCanvas *c): FbeIO(NULL)
{
   _doit= 1;
   _worms= new TClonesArray("TPolyLine");
   _lines= new TClonesArray("TLine");
   _m= (short*)malloc(100*sizeof(short));
   _cmap= (short*)malloc(MXXMDF*sizeof(short));

   _image= 0;
   _timg= 0;
   _aimg= 0;
   _c1= c;
   _aLine= (unsigned char*)malloc(1);
   _all= (unsigned char*)malloc(1);
   DEBUG(printf(" --- FbePlotgeom constructor\n");)
}


void FbePlotgeom::doPlotgeom(pgmaxS *p, int mode, int bin)
{
   wormS *wp;
   int i,j,k;

   _llx= _x0= p->X0 + p->YAXLEN * p->TYX;
   _lly= _y0= p->Y0 + p->YAXLEN * p->TYY;
   _llz= _z0= p->Z0 + p->YAXLEN * p->TYZ;
   _urx= _x1= p->X0 + p->XAXLEN * p->TXX;
   _ury= _y1= p->Y0 + p->XAXLEN * p->TXY;
   _urz= _z1= p->Z0 + p->XAXLEN * p->TXZ;

   _t0= _x0 * p->TYX + _y0 * p->TYY + _z0 * p->TYZ;
   _t1= _x1 * p->TYX + _y1 * p->TYY + _z1 * p->TYZ;
   _r0= _x0 * p->TXX + _y0 * p->TXY + _z0 * p->TXZ;
   _r1= _x1 * p->TXX + _y1 * p->TXY + _z1 * p->TXZ;

   _x0= _r0; _x1= _r1; if (_r1<_r0) { _x0= _r1; _x1= _r0; }
   _y0= _t0; _y1= _t1; if (_t1<_t0) { _y0= _t1; _y1= _t0; }

   int Inw= 0;
   DEBUG(printf("\n ---> just before callinf scan1time\n\n");)
   wormS *w= scan1time(&Inw,mode,bin);
   if (w==(wormS*)NULL) { _nrOfWorms= 0; _nrOfLines= 0; return; }
   TClonesArray &ptr= *_worms;

   for (j=0;j<Inw;j++) { wp= &w[j];
                         for (i=0;i<wp->n;i++) { wp->x[i]+= _x0; wp->y[i]+= _y0; }
//                      --------------------------------------------
                         new(ptr[j]) TPolyLine(wp->n,wp->x,wp->y); }

   _nrOfWorms= Inw;

   al= getLines();
   _m= (short*)realloc(_m,al->n*sizeof(short));

   DEBUG(printf(" - _pXYZ->lwmhol %i\n",p->lwmhol);)

   double xl,xk,yl;
   TClonesArray &zgr= *_lines;

   k= 0;
   i= 0;
   while (i<al->n) { j= i; while (al->m[j]>0) j++;
                     yl= al->c[j]+_y0;
                     xk= al->c[i++]+_x0;
                     while (i<j) { _m[k]= al->m[i-1];
                     xl= al->c[i++]+_x0;
                     new(zgr[k++]) TLine(xk,yl,xl,yl);
                     xk= xl;
                   }
        i++;
      }
// ------------------------------------------------------------- paint ---
// while (i<al->n) { j= i; while (al->m[j]>0) j++; i++;
//                   while (i<j) { _m[k]= al->m[i-1]; i++; k++; }
//                   i++;
//                 }

   _nrOfLines= k;
   DEBUG(printf(" ---> %i\n",k);)

   int maxM=MXXMDF;
   for (i=0;i<maxM;i++) _cmap[i]= 0;

   for (k=0;k<_nrOfLines;k++) { if (_m[k]>=maxM) printf(" ===> _m[k]>maxM %i/%i\n",k,_m[k]);
                                _cmap[_m[k]]= 1; }
   // printf(" -b-> %i\n",maxM);

   _nMedia= 1;
   for (i=0;i<maxM;i++)
      if (_cmap[i]>0) _cmap[i]= _nMedia++;

   DEBUG(printf(" -debug-> nw %i X1 %f nrScanLines %i mode %i\n", Inw,p->YAXLEN,p->lwmhol,mode);)
}


//
// --- this is our constructor in case we read from a static PLOTGEM.STORE type file
//     very deprecated way to do things - this file only knows it's one view, in case the
//     user likes to look at a different view a new file is required
//    ------------------------------------------------------------------------------------------------
//

FbePlotgeom::FbePlotgeom(const char *aName): FbeIO(aName)
{

    int i,j,k, bytesRead,bytes, words[3],nrOfWorms,wormLength, swap=0;
    static char title[81];
    float coors[16], *arr,*x,*y;

    _themax= 0;

    _worms= new TClonesArray("TPolyLine");
    TClonesArray &ptr= *_worms;

    _lines= new TClonesArray("TLine");
    _m= NULL;

    _fd= open(_filename,O_RDONLY);

    if (_fd<0) _status= 1;
    else {
           DEBUG(printf(" -debug-> FbePlotgeom()  _fd %i\n",_fd);)
           _endianess= FBE_littleEndian;

//     read first fortran record length, should ==80 and so indicate which endianess we are
//     looking at - we should find 1/2 strings here a possible tag to indicate compressed style
//     information and a title (given to the PLOTGEOM option card
//    ------------------------------------------------------------------------------------------------

           bytesRead= read(_fd,&bytes,4);

           DEBUG(printf(" -debug-> FbePlotgeom() bytesRead _%i_\n",bytesRead);)

           if (bytes==CURRENT_PLOTGEOM_FILE_TYPE) _usrDataType= FBE_plotgeom;
             else {
                    bytes= swab32(bytes);
                    if (bytes==CURRENT_PLOTGEOM_FILE_TYPE) { _endianess= FBE_bigEndian;
                                                             _usrDataType= FBE_plotgeom; }
                      else { DEBUG(printf(" -debug-> FbePlotgeom() aerger with file (ascii?)\n");)
                             _status= 2; }
                  }
         }

    if (_status==0) {

           bytesRead= read(_fd,title,bytes);
           i= bytesRead-1; while (title[i]==' ') i--; title[i+1]= '\0';
           bytesRead= read(_fd,&bytes,4);

           if (strstr(title,"COMPRESSED")) { bytesRead= read(_fd,&bytes,4);
                                             bytesRead= read(_fd,title,bytes);
                                             i= bytesRead-1; while (title[i]==' ') i--; title[i+1]= '\0';
                                             bytesRead= read(_fd,&bytes,4); }
           strcpy(_title,title);

//     read through PLOTGEOM input parameters 'lower left' (x/y/z), 'upper right' (x/y/z),
//     Tx, Ty and x/y aspect ratio
//     turn 'worms' into absolute coordinates to simplify overlay on actual histogram
//    ------------------------------------------------------------------------------------------------


           bytesRead= read(_fd,&bytes,4);
           bytesRead= read(_fd,coors,56);
           bytesRead= read(_fd,&bytes,4);

DEBUG(printf(" o> %9.5f %9.5f %9.5f \n",coors[0],coors[1],coors[2]);)
DEBUG(printf(" o> %9.5f %9.5f %9.5f  %9.5f %9.5f %9.5f\n",coors[6],coors[7],coors[8],coors[9],coors[10],coors[11]);)
DEBUG(printf(" o> %9.5f %9.5f  \n",coors[13],coors[12]);)

           _llx= _x0= coors[0]+coors[13]*coors[6];
           _lly= _y0= coors[1]+coors[13]*coors[7];
           _llz= _z0= coors[2]+coors[13]*coors[8];
           _urx= _x1= coors[0]+coors[12]*coors[9];
           _ury= _y1= coors[1]+coors[12]*coors[10];
           _urz= _z1= coors[2]+coors[12]*coors[11];

           oX= (int)(coors[6]+2*coors[7]+3*coors[8])-1;
           oY= (int)(coors[9]+2*coors[10]+3*coors[11])-1;

           _t0= _x0*coors[6]+_y0*coors[7]+_z0*coors[8];
           _t1= _x1*coors[6]+_y1*coors[7]+_z1*coors[8];
           _r0= _x0*coors[9]+_y0*coors[10]+_z0*coors[11];
           _r1= _x1*coors[9]+_y1*coors[10]+_z1*coors[11];

           DEBUG(printf(" ---> %g %g %g\n",_llx,_lly,_llz);)
           DEBUG(printf(" ---> %g %g %g\n",_urx,_ury,_urz);)

           _x0= _r0; _x1= _r1; if (_r1<_r0) { _x0= _r1; _x1= _r0; }
           _y0= _t0; _y1= _t1; if (_t1<_t0) { _y0= _t1; _y1= _t0; }

           DEBUG(printf(" ---> %g %g %g %g\n",_x0,_x1,_y0,_y1);)

//     actually read through the 'worms'
//     scaling of axis is not taken care of here
//    ------------------------------------------------------------------------------------------------

           k= 0;
           while (read(_fd,&bytes,4)>0) {
                bytesRead= read(_fd,words,8);
                bytesRead= read(_fd,&bytes,4);
                nrOfWorms= words[0];
                //DEBUG(printf(" -debug-> words %i %i\n",words[0],words[1]);)

                for (i=0;i<nrOfWorms;i++) {
                   bytesRead= read(_fd,&bytes,4);
                   bytesRead= read(_fd,words,12);
                   bytesRead= read(_fd,&bytes,4);
                   wormLength= words[2];
                   //DEBUG(printf(" -debug-> words %i %i %i\n",words[0],words[1],words[2]);)

                   bytesRead= read(_fd,&bytes,4);
                   if (wormLength*2*4 != bytes) { DEBUG(fprintf(stderr," +++> aerger in plotgeom file\n");)
                                                  _status= 3; }
                   arr= (float*)malloc(bytes);
                   bytesRead= read(_fd,arr,bytes);

                   x= (float*)malloc(bytes);
                   y= (float*)malloc(bytes);
                   if (bytes>_themax) _themax= bytes;

                   for (j=0;j<wormLength;j++)
                      if (swap==1) { y[j]= arr[2*j]+_x0; x[j]= arr[2*j+1]+_y0; }
                         else { x[j]= arr[2*j]+_x0; y[j]= arr[2*j+1]+_y0; }

//               --------------------------------------------
                   new(ptr[k++]) TPolyLine(wormLength,x,y);

                   free(arr);
                   free(x);
                   free(y);
                   bytesRead= read(_fd,&bytes,4); } }

           _nrOfWorms= k;

           if (_fd>0) close(_fd);
         }
}


//
// --- well, we had some trouble here recently (sep'06 11-03:11)
//     in our static PLOTGEOM.STORE case we do have to setup _lines/_m even though we don't
//     use those then - otherwise the destructor crashes more or less randomly
//    ------------------------------------------------------------------------------------------------

FbePlotgeom::~FbePlotgeom()
{
    DEBUG(printf(" -debug-> ~FbePlotgeom() -about to call .Delete on _worms-\n");)
    printf(" ---> ~FbePlorgeom() -- this should not happen\n");
    if (_worms) { delete _worms; _worms= NULL; }
    if (_lines) { delete _lines; _lines= NULL; }
    if (_m) { free(_m); _m= NULL; }
}


//
// --- working our overlay that just draws the poly lines
//     no checking on anything (clipping done by TPad), data for now read from PLOTGEOM.STORE file
//     or generated through our pgmdrv wrapper
//     swapping asked for according to the histogram information from within UpdateHistogram
//    ------------------------------------------------------------------------------------------------
//

void FbePlotgeom::overlay(int swap, int col)
{
   TPolyLine *pl,*tmp;
   TClonesArray &ptr=*_worms;

   for (int k=0;k<_nrOfWorms;k++) { tmp= (TPolyLine*)ptr[k];
                                    if (swap) pl= new TPolyLine(tmp->GetN(),tmp->GetY(),tmp->GetX());
                                      else pl= tmp;
                                    pl->SetLineColor(col); pl->Draw("SAME"); }
}


// void FbePlotgeom::plot(int swap) { printf(" -XXX- plot() removed\n"); }
void FbePlotgeom::plot(int swap)
{
TLine *l,*tmp;
TClonesArray &ptr=*_lines;

for (int k=0;k<_nrOfLines;k++) { tmp= (TLine*)ptr[k];
                                 if (swap) l= new TLine(tmp->GetY1(),tmp->GetX1(),tmp->GetY2(),tmp->GetX2()); else l= tmp;
                                 l->SetLineColor( _colors[      _m[k] ]->GetNumber() ); l->Draw("SAME"); }
                             //  l->SetLineColor( _colors[_cmap[_m[k]]]->GetNumber() ); l->Draw("SAME"); }
}



//
// --- mar'07 10-15:17
//     just a try for now, lets make a real picture rather than many TLines (maybe make this semi
//     transparent in addition) - push a raw XImage -> Pixmap -> TImage
//     flawed somehow, fixed by writing/reading .gif file
//
// --- mar'07 13-17:45
//     we need a cleaner way to fill our raster image
//     rather than using XImage let's try and create an in-memory .xpm so to skip the XImage step
//     this because otherwise we would need to know/learn how to deal with changing display depth
//     (raw code only worked for >24bit color depth)
//     let us use xpmCreatePiaxmapFromData(char**,nx,ny) and check what happens for 16bpp
//

void FbePlotgeom::paint(int swap,int nx,int ny)
{
   int i,j,k,ky, r,g,b, n=((nx>ny)?nx:ny)+1;
   ULong_t cp;
   double dx=(_x1-_x0)/n,dy=(_y1-_y0)/n;

   printf(" ---> paint  %i %i %i (depth %i)\n",swap,nx,ny, gVirtualX->GetDepth());
   printf(" --->        %f %f %f %f - %12.8f %12.8f - %i\n",_x0,_x1,_y0,_y1, dx,dy, n);

   unsigned char *pl;
   _aLine= (unsigned char*)realloc(_aLine,n);
   _all= (unsigned char*)realloc(_all,n*n);
   int acolor,from,to;

   FILE *fp=fopen("/tmp/o.xpm","w");
   fprintf(fp,"/* XPM */\n");
   fprintf(fp,"static char * c_xpm[] = {\n");
   fprintf(fp,"\"%i %i %i %i\",\n",n,n,_nMedia,1);
   for (int nc=0;nc<_nMedia%100;nc++) { cp= _colors[nc]->GetPixel(); b= cp%256; g= (cp>>8)%256; r= (cp>>16)%256;
                                        fprintf(fp,"\"%c\tc #%02x%02x%02x\",\n",(unsigned char)nc+'0',r,g,b); }

   ky= 0;
   i= 0;
   while (i<al->n) { j= i; while (al->m[j]>0) j++;
                     from= 0;
                     i++;
                     pl= &_all[ky*n];
                     while (i<j) { acolor= _cmap[al->m[i-1]];
                                   to= int(round(al->c[i++]/dx));
                                   for (k=from;k<to;k++) *pl++= (unsigned char)acolor+'0';
                                   from= to;
                                 }
                     ky++;
                     i++;
                   }

   for (j=0;j<n;j++) { pl= &_all[(n-1-j)*n];
                       for (i=0;i<n;i++) _aLine[i]= pl[i];
                       fprintf(fp,"\"%s\"",_aLine); if (j<n) fprintf(fp,",\n"); }

   fprintf(fp,"};\n");
   fclose(fp);

   if (_timg) { delete _timg; _timg= 0; } _timg= TImage::Open("/tmp/o.xpm");
   _timg->SetEditable(kTRUE);
   _timg->SetConstRatio(kFALSE);
   _c1->cd();
   _timg->Draw();
}

