
//
//    mar'04
//    jan'05 - version v.03
//    (feb'07 19-02:07) - version v.07a
//                                                                                 the FLEUR project
// -------------------------------------------------------------------------------------------------
//                                                                                anton.empl@cern.ch
// --- FLUKA user back-end project -Fbe-
//     this class is to be used in context of the FLUKA GUI project
//     we attach the plotgeom file and read in the 'worms'
//     alternatively we extract the geometry information from the FLUKA input file
//     and generate the scan grid on demand
//
//     may'06 10-01:46
// --- added a new way to get materials colored in
//
//     mar'07 09-22:13
// --- redesign use of this class
//     let's mae instances for all 3 views but keep them alive
//     push action into doPlotgeom() instead where we try to recycle all memory
//


#ifndef __MFBEPLOTGEOM_H
#define __MFBEPLOTGEOM_H

#define DEBUG(arg)      {if (getenv("DEBUG")) arg;}


#include <TROOT.h>
#include <TSystem.h>
#include <TObject.h>
#include <TClonesArray.h>
#include <TArrayI.h>
#include <TArrayC.h>
#include <TH2C.h>
#include <TColor.h>
#include <TStyle.h>
#include <TLine.h>
#include <TPolyLine.h>
#include <TString.h>

#include <TImage.h>
#include <TPad.h>
#include <TCanvas.h>
#include <TGX11.h>
#include <TVirtualX.h>

#include <fcntl.h>
#include <math.h>


#include "FbeIO.h"
#include "scanGeometry.h"

extern "C" { wormS *scan1time(int*,int,int); }
extern "C" { aLS *getLines(); }

#define CURRENT_PLOTGEOM_FILE_TYPE 80


class FbePlotgeom : public FbeIO, public TObject
{

private:

double         _x0,_y0,_z0,_x1,_y1,_z1, _r0,_r1,_t0,_t1;  //
double         _llx,_lly,_llz,_urx,_ury,_urz;             //
int            _nrOfWorms,_themax, oX,oY;                 //
char           _title[256];                               //

TClonesArray   *_worms,*_lines;                           //

int            _doit;                                     //
int            _nrOfLines;                                //
int            _colorsAreMapped;                          //  -- jul'06 04-10:52
int            _nMedia;                                   //
short          *_m;                                       //! make member non persistent - now what's this
short          *_cmap;                                    //!
TColor         *_colors[100];                             //

double         _buf[17],_abuf[13];                        //

XImage         *_image;                                   //
TImage         *_timg;                                    //
TImage         *_aimg;                                    //
GContext_t     _gContext;                                 //
Window_t       _wID;                                      //
TCanvas        *_c1;                                      //
aLS            *al;                                       //


public:

FbePlotgeom(const char *aName);
FbePlotgeom(TCanvas *id);
virtual ~FbePlotgeom();

void doPlotgeom(pgmaxS *XYZ, int mode=0, int bin=0);
inline int reDoit() { return(_doit); }
inline void reset() { _doit= 1; }

int status() { return (_status); }

short *getColors(int *n) { *n= _nMedia; return _cmap; }
void mapColors(TColor *m[]) { int i; for (i=0;i<100;i++) _colors[i]= m[i]; }
void plot(int swap=0);
void overlay(int swap=0, int col=15);

void paint(int swap=0,int nx=33,int ny=33);
void help() { printf(" -help-> %p\n",_c1); }

inline char *getTitle() { return (_title); }
inline int getX() { return (oX); }
inline int getY() { return (oY); }
inline void get2D(double *x0, double *y0, double *x1, double *y1)
                  { *x0= _x0; *y0= _y0; *x1= _x1; *y1= _y1; }
inline void get3D(double *x0, double *y0, double *z0, double *x1, double *y1, double *z1)
                  { *x0= _llx; *y0= _lly; *z0= _llz; *x1= _urx; *y1= _ury; *z1= _urz; }

ClassDef(FbePlotgeom,1)

};


#endif
