
//
//    (feb'07 19-02:07) - version v.07a
//                                                                                 the FLEUR project
// -------------------------------------------------------------------------------------------------
//                                                                                anton.empl@cern.ch
//                                                                   (Katey Covin, Matt LeBourgeois)
// --- FLUKA user back-end project -Fbe-
//     see header file (FbeUsrbin.h) for comments
//


#include "FbeUsrbin.h"
#include<iostream>
using namespace std;
ClassImp(FbeUsrbin)



//
//
// --- here's our constructor
//     we are called through FbeUsrbinFile and handed down the per binning information which still
//     needs to be refined - deal with casting a few (float)s onto (int)s and try to clear out extra
//     digits introduced here since we read single precision fortran binary data
//    ------------------------------------------------------------------------------------------------


FbeUsrbin::FbeUsrbin(int ctr,int aNr,char aName[10],int aType,int aScore,float aArr[16],int aFrom,int aBytes)
{
   int i,j;

   _bins.Set(3);
   _limits.Set(9);
   zCurrent.Set(6);
   combCurrent.Set(6);

//     these give the location for the data of this binning in the file and indicate we 
//     actualy perform a read already
//    ------------------------------------------------------------------------------------------------

   _data= 0;
   _dataNotYetRead= 1;
   _from= aFrom;
   _howManyBytes= aBytes;

//     some more general header information per binning, FLUKA specific
//     subtract 1 from aNr since it's a FORTRAN counter in origin
//    ------------------------------------------------------------------------------------------------

   _index= ctr;
   _flukaID= aNr-1;
   strcpy(_theName,aName);
   _type= aType;
   _score= aScore;

//     re-assamble the (int) number of bins information - kept as (float) in aArr[]
//     and slightly align the limits by skipping the (float) bin
//     in addition we setup our integer limits to project the 3D information
//     don't trust the handed bin size
//    ------------------------------------------------------------------------------------------------

   _bins[0]= *(int*)&aArr[2];
   _bins[1]= *(int*)&aArr[6];
   _bins[2]= *(int*)&aArr[10];

   j= 0;
   for (i=0;i<9;i++) { if (0==(j+2)%4) j++; _limits[i]= push2double(aArr[j++]); }

// --- sep'06
//     alfredo used a test example where zmax<zmin
//     we basically bomb in Matt's NumberBin() thingy
//     place safety catch here

   for (i=0;i<3;i++)
      if (_limits[3*i+1]-_limits[3*i]<0) { printf(" --- min/max problem: %f %f (%i)\n",_limits[3*i+1],_limits[3*i],i);
                                           double t= _limits[3*i+1]; _limits[3*i+1]= _limits[3*i]; _limits[3*i]= t; }
//
// ---

   for (i=0;i<3;i++) _limits[3*i+2]= (_limits[3*i+1]-_limits[3*i])/_bins[i];


//     plus we also have some extra FLUKA information to carry over
//    ------------------------------------------------------------------------------------------------

   _lntzer= *(int*)&aArr[12];
   _bkusbn= push2double(aArr[13]);
   _b2usbn= push2double(aArr[14]);
   _tcusbn= push2double(aArr[15]);

   _withStatisticalErrorEstimate= 0;
   _Sdata= 0;

   _rawMin= 1e130;
   _rawMax=-1e130;
   _rphi= (_type==1 ||_type==11);  // -ae- && fabs(_limits[3]+acos(-1.))<.1e-5 && fabs(_limits[3+1]-acos(-1.))<.1e-5;
   _regions= _type==1 || _type==11;
   _supported= _type==0 || _type==10 || _type==1 || _type==11;
   _rots= 0;
   
   _rphiz= 0;
   if (_rphi) _rphiz= _bins[1]==1;


//     some more initialisation in ocntext of the GUI
//    ------------------------------------------------------------------------------------------------

   _wasRebinned= 0; _rebin= 0;

   for (int k=0;k<3;++k) fXYZGraphed[k]= 0; 
   for (int k=0;k<6;++k) { zCurrent[k]= -1; combCurrent[k] = 1; }


//
// --- mar'06 05-14:43
//     very C like, should rather be a class I guess
//     should have one of these GUI 'states' per binning

   for (int i=0;i<3;i++) { _Rem.Title[i]= (char*)malloc(256);
                           _Rem.XLabels[i]= (char*)malloc(256);
                           _Rem.YLabels[i]= (char*)malloc(256);
                           _Rem.ZLabels[i]= (char*)malloc(256);
                           _Rem.Scored[i]= (char*)malloc(256);

                           strcpy(_Rem.Title[i]," -nix- ");
                           _Rem.Scale[i]= 1;
                           _Rem.appliedScale[i]= 1;
                         }

   

}


//
// --- sep'06 19-03:50
//     probably just a start to clean up
//

FbeUsrbin::~FbeUsrbin() { for (int i=0;i<3;i++) { if (fXYZGraphed[i]) fHistXYZ[i]->Delete(); } }


//
// --- this first method is actually declared protected so that only FbeUsrbinFile which is
//     chosen to be a friend class can access it
//    ------------------------------------------------------------------------------------------------

void
FbeUsrbin::addStatistics(int aFrom, int aBytes)
{
    _withStatisticalErrorEstimate= 1;
    _Sfrom= aFrom;
    _ShowManyBytes= aBytes;
}



//
// --- the file contains all numbers in single precision from a FORTRAN binary write
//     much better to keep the important ones (the ones we'll do gymnastics with that is) in double
//     so we need/want to be careful about extra random digits  < only trust 7 digits >
//    ------------------------------------------------------------------------------------------------

double
FbeUsrbin::push2double(float xIn)
{

float x;
double keepSign,log10x,pof10,cutx,shiftx,xOut=0, zero=.1e-130,ten=10,digits=1000000;

if (fabs(xIn)<zero) return (0);

keepSign= fabs(xIn)/xIn;
x= fabs(xIn);

if (fabs(xIn)>zero) { log10x= log10(x);
                      pof10= floor(log10x);
                      shiftx= pow(ten,pof10);

                      cutx= round(x/shiftx*digits);
                      xOut= cutx/digits*shiftx *keepSign;
                    }
return (xOut);
}



//
// --- try and find the closest bin edge from where we are in the 'view' coordinate
//    ------------------------------------------------------------------------------------------------

int
FbeUsrbin::edge(int view, double value, int upper) const
{
int theEdge;
double fraction,d,low=_limits[3*view],high=_limits[3*view+1],dxyz=_limits[3*view+2];

if (!_supported) return(-1);

if (value>=low && value<=high) { d= (value-low)/dxyz;
                                 theEdge= (int)d;
                                 fraction= d-theEdge;
                                 if (fraction>.5 || (fraction>=.5 && upper)) theEdge++;  }
  else theEdge= (value>high) ? _bins[view]:0;

return (theEdge);
}




// ---------------------------------------------------------------------------------------------------
// -ae-
//
//     jan'05 05-11:55
// --- here our new way to fill the 2D information for our histogram
//     want this to be flexible so that we can handle r/phi/z information also and to fill both
//     content and error (if present) arrays - follow former FLUKA ideas, ie transform data and
//     error back to be absolute rather than density and after all the gymnastics we return to
//     density - also turn relative error into absolute
//
//     this will be exercised whenever we need to integrate over a new Z range (ie 3rd coordinate)
//     we'll keep the arrays with their full x/y ranges regardless of viewing or re-binning
//    ------------------------------------------------------------------------------------------------
//

void FbeUsrbin::project2D(int view, int lo, int hi)
{
int x,y,z, rxyz[3],ix=view%3,iy=(view+1)%3,iz=(view+2)%3, m,n, nArr=_bins[ix]*_bins[iy];
double v,*arr,*err=NULL,e;
double dx,dy,dz,dv,volumeOfColumn;    // zero=1e-70;

DEBUG(printf(" -project2D- ( %3i )\n",_index);)

if (!hi) hi=_bins[iz];

arr= (double*)calloc(nArr,sizeof(double));
if (_withStatisticalErrorEstimate) err= (double*)calloc(nArr,sizeof(double));

for (x=0;x<_bins[ix];x++) { rxyz[ix]= x; dx= dV(ix,x);
   for (y=0;y<_bins[iy];y++) { rxyz[iy]= y; dy= dV(iy,y); volumeOfColumn= 0;
                               m= x+y*_bins[ix];
            for (z=lo;z<hi;z++) { rxyz[iz]= z; dz= dV(iz,z);
                                  n= rxyz[0]+rxyz[1]*_bins[0]+rxyz[2]*_bins[1]*_bins[0];
                                  volumeOfColumn+= dv= dx*dy*dz;
                                  arr[m]+= _data[n]*dv;
                                  if (_withStatisticalErrorEstimate) { e= _Sdata[n]*_data[n]*dv; err[m]+= e*e; }
                                }
      arr[m]/= volumeOfColumn;
      if (_withStatisticalErrorEstimate) err[m]= sqrt(err[m])/volumeOfColumn;

      v= fabs(arr[m]);   // --- see pawlevbin.f
      if (v>_rawMax) _rawMax= v; else if (v<_rawMin) _rawMin= v;
     }}

_cont[view].Adopt(nArr,arr);
if (_withStatisticalErrorEstimate) _err[view].Adopt(nArr,err);
}



//
//     jan'05 20-03:21
// --- in case the user wants to rebin a histogram, a depricated feature here, we need to change
//     histogram properties - we fill yet another set of arrays with the exact dimensions asked for
//     everytime starting from the above full range 2D arrays
//     these arrays are the ones to be filled into the root histogram objects
//    ------------------------------------------------------------------------------------------------
//

void FbeUsrbin::rebin2D(int view, int lx, int hx, int Bx, int ly, int hy, int By)
{
int i,ii,x,j,jj,y,k,kk,kx,ky, Nx=(hx-lx),Ny=(hy-ly),nx=Nx/Bx,ny=Ny/By, ix=view%3,iy=(view+1)%3;
double dv,dx,dy,e,dVolume;

fXYZMat[ix].Set(nx*ny);
fXYZStatMat[ix].Set(nx*ny);

DEBUG(printf(" -rebin2D- -- %3i %3i %3i - %3i %3i %3i\n",Nx,nx,Bx,Ny,ny,By);)

for (i=0,x=lx;i<nx;i++,x+=Bx)
   for (j=0,y=ly;j<ny;j++,y+=By) { k= i+j*nx;  fXYZMat[ix][k]= fXYZStatMat[ix][k]= dVolume= 0;

      for (ii=0;ii<Bx;ii++) { kx= x+ii; dx= dV(ix,kx);
         for (jj=0;jj<By;jj++) { ky= y+jj; dy= dV(iy,ky);

            kk= kx+_bins[ix]*ky;
            dVolume+= dv= dx*dy;
            fXYZMat[ix][k]+= _cont[ix][kk]*dv;
            if (_withStatisticalErrorEstimate) { e= _err[ix][kk]*dv; fXYZStatMat[ix][k]+= e*e; }

         }}
   fXYZMat[ix][k]/= dVolume;
   if (_withStatisticalErrorEstimate) fXYZStatMat[ix][k]= sqrt(fXYZStatMat[ix][k])/dVolume;
   }
}



//
// --- just get the correct dV for x/y/z or r/phi/z for conversion from/to densities
//     this method gets used by the above two integration routines
//     actually we prepare the correct line element dx per coordinate
//    ------------------------------------------------------------------------------------------------
//

double FbeUsrbin::dV(int xyz, int n)
{
double dv=_limits[xyz*3+2];
if (_rphi) { if (xyz==0) dv= (2*n+1)*dv*dv;      // *pi    <--- r
               else if (xyz==1) dv*= .5; }       // /pi    <--- phi
return (dv);
}



//
// --- here we simply push every value from our 2D array(s) into the actual root histogram
//     could/should be a trivial thing, don't know how to do it in one step though
//    ------------------------------------------------------------------------------------------------
//     swap is now handled byt THistPainter (feb'06 23-13:27)
//

void FbeUsrbin::fill2D(TArrayD cont, TArrayD err, int nx, int ny, FbeHist *fbeH)
{
  int i,j,k;
  double low=1e99,high=1e-99;

  DEBUG(printf(" -fill2D- nx %i ny %i\n",nx,ny);)

  for (i=1;i<=nx;i++)
     for (j=1;j<=ny;j++)
     {
        k= i-1+nx*(j-1);
        if (cont[k]>high) high= cont[k];
        if (cont[k]>0 && cont[k]<low) low= cont[k];

        fbeH->SetBinContent(i,j,cont[k]);
        if (_withStatisticalErrorEstimate) fbeH->SetBinError(i,j,err[k]);
     }
  fbeH->minimum(low);
  fbeH->maximum(high);
}

//
// ---------------------------------------------------------------------------------------------------




//
//    return a TH2F object here
//    only fill cropped to bin edges in 'XY' even though arr/err have full range
//    'Z' integration already happened
//    default full range histogram is implemented
// ---------------------------------------------------------------------------------------------------
//

FbeHist*
FbeUsrbin::hist(int view)
{
int ix=view%3,iy=(view+1)%3;
double x0=_limits[3*ix],x1=_limits[3*ix+1],y0=_limits[3*iy],y1=_limits[3*iy+1];

DEBUG(printf(" -wrap- %3i\n",view);)
return (hist(view,x0,x1,y0,y1));
}



//
//     jan'05 22-03:50
// --- re-worked central method to perfom histogramming task
//
//     problem: SetBins() isn't the same as SetBinsLengs() + GetXaxis()->Set() WHY?
//    ------------------------------------------------------------------------------------------------
//     at this point rebin() seem to work even though it's all still fragile
//     and needs more testing - the logic here is pretty messy
//     look also at FlukaMainFrame::  ChangeRanges()/ReflectRanges()/OurStyle()
//                                                                                     jan'05 27-05:12
//


FbeHist*
FbeUsrbin::hist(int mode, double xx0,double xx1, double yy0,double yy1,double zz0,double zz1,int xcomb,int ycomb)
{

int ix=mode%3,iy=(mode+1)%3,iz=(mode+2)%3, xLo=edge(ix,xx0,0),xHi=edge(ix,xx1,1),
                        yLo=edge(iy,yy0,0),yHi=edge(iy,yy1,1), zLo=edge(iz,zz0,0),zHi=edge(iz,zz1,1);
double x0,x1,y0,y1;
int Nx=xHi-xLo,nx=Nx/xcomb,Ny=yHi-yLo,ny=Ny/ycomb;
char name2[2]={"x"},name[80],*nn[3]={"XY","YZ","ZX"}; sprintf(name,"view_%s",nn[ix]);

DEBUG(printf(" -hist- ( %3i ) -- %3i %f %f %3i %f %f\n",_index, Nx,xx0,xx1,Ny,yy0,yy1);)

if (xLo>=xHi || yLo>=yHi || zLo>=zHi) { printf(" -o- "); return(hist(mode)); }


//
// --- first we create or rework our root histogram here, always for the entire x/y range
//     and integrated from zLo to zHi - we skip this step if we are called to prepare a rebinned
//     histogram - if however we were rebinned already and switch to another histogram without
//     the proper button pushed we need to revert to the original bin/range situation

FbeHist *h=NULL;

if (!_rebin) {

  nx= _bins[ix]; x0= _limits[3*ix]; x1= _limits[3*ix+1];
  ny= _bins[iy]; y0= _limits[3*iy]; y1= _limits[3*iy+1];

//
// --- if the histogram does not yet exist - let's make our default range max integrated one
//     then we have a histogram h

  if (!fXYZGraphed[ix]) { TH2 *c= (TH2*)gROOT->FindObject(name); if (c) delete c;
                          fHistXYZ[ix]= new FbeHist(name,name,nx,x0,x1,ny,y0,y1);
                          fHistXYZ[ix]->rphi(_rphi); }
  h= fHistXYZ[ix];

//
// --- check if we need to rework it
//

  if (zCurrent[2*iz]!=zLo || zCurrent[2*iz+1]!=zHi || _wasRebinned) {


    if (_wasRebinned) { h->SetBinsLength((nx+2)*(ny+2));
                        h->GetXaxis()->Set(nx,x0,x1);
                        h->GetYaxis()->Set(ny,y0,y1); }

     else if (zCurrent[2*iz]!=zLo || zCurrent[2*iz+1]!=zHi) project2D(mode,zLo,zHi);

    h->Reset();
    h->SetName(name2);

// -------------------------------------
    fill2D(_cont[ix],_err[ix],nx,ny,h);

    fHistXYZ[ix]= h;
    zCurrent[2*iz]= zLo; zCurrent[2*iz+1]= zHi;

    _wasRebinned= 0;
    fXYZGraphed[ix]= true;

    return fHistXYZ[ix]; }


  } // --- (_rebin) ---

  else {
         DEBUG(printf(" <+> rebin <+>\n");)
         h= fHistXYZ[ix];
         if (combCurrent[2*ix]!=xcomb || combCurrent[2*ix+1]!=ycomb) {

           rebin2D(mode,xLo,xHi,xcomb,yLo,yHi,ycomb);

           h->SetBinsLength( (nx+2)*(ny+2) );
           h->GetXaxis()->Set(nx,xx0,xx1);
           h->GetYaxis()->Set(ny,yy0,yy1);

           h->Reset();
           h->SetName(name2);

           fill2D(fXYZMat[ix],fXYZStatMat[ix],nx,ny, h);

           fHistXYZ[ix]= h;
           combCurrent[2*ix]= xcomb; combCurrent[2*ix+1]= ycomb;
           _wasRebinned= 1;

           return fHistXYZ[ix]; }
       }   

return fHistXYZ[ix];
}



//
// ---

FbeHist *FbeUsrbin::GetHist(int view)
{
	if(!fXYZGraphed[view%3])
	{
		fHistXYZ[view%3] = hist(view%3);
		if(fHistXYZ[view%3]) fXYZGraphed[view%3] = true;
	}
	fHistXYZ[view%3]->SetName(_withStatisticalErrorEstimate?"S":"h");
	return fHistXYZ[view%3];
}



//
// ---

void FbeUsrbin::triggerFill2D(int mode, int nx, int ny)
{
    printf(" -triggerFill2D-\n");
    int ix=mode%3;
    fill2D(_cont[ix],_err[ix],nx,ny,fHistXYZ[ix]);
}


