
//
//    (feb'07 19-02:07) - version v.07a
//                                                                                 the FLEUR project
// -------------------------------------------------------------------------------------------------
//                                                                                anton.empl@cern.ch
//                                                                   (Katey Covin, Matt LeBourgeois)
// --- FLUKA user back-end project -Fbe-
//     class to keep usrbin information (header/data) per binning
//     usrbin information is given in single precision (even though it is computed within FLUKA in
//     double precision) as FORTRAN binary output
//     protected data members are visible to and manipulated by the FbeUsrbinFile class as well
//     taken all built-in variables back to regular C/C++ types
//
//     sep'06 18-12:15
//     trouble with USRBIN data in case of 'missing' binnings, ie they were written to a different
//     file - here our indexof() which was set to fluka counting, of course did not reflect
//     position of histogram in our own list here, which is restricted to a single file
//     introduce _flukaID in addition to _index
//

#ifndef __FBE_USRBIN_H
#define __FBE_USRBIN_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define DEBUG(arg)      {if (getenv("DEBUG")) arg;}

 
#include <TObject.h>
#include <TString.h>
#include <TArrayD.h>

#include "FbeHist.h"

#include <math.h>


//     mar'06 05-15:23
// --- to remember our state for other views
//     (here is Tab3)

typedef struct { double Scale[3],appliedScale[3], Anchor[3],RoofTop[3], AspectRatio[3];
                 char   *Title[3],*XLabels[3],*YLabels[3],*ZLabels[3],*Scored[3];
                 int    selected;
                 int    Histogram[3],Geometry[3],Aspect[3],
                        Swap[3],Normalize[3],EnableColorPalette[3],FLukaScheme[3],Logarithmic[3],CPD[3]; } fRemS;
// ---



class FbeUsrbin : public TObject
{

friend class FbeUsrbinFile;


private:

int             _flukaID,_index,_type,_score;                   //
char            _theName[10],_theType[32],_theScore[32];        //
int             _rphi,_rphiz,_regions,_supported;               //
double          _rawMin,_rawMax;                                //
double          _bkusbn,_b2usbn,_tcusbn;                        //
int             _lntzer;                                        //
int             _rots;                                          //

TArrayD         _limits;                                        //
TArrayI         _bins;                                          //

TArrayD         _cont[3];                                       //
TArrayD         _err[3];                                        //

int             _rebin,_wasRebinned;                            //


TArrayI	        zCurrent,combCurrent;                           // array to remember integration limits over 3rd coord
TArrayD	        fXYZMat[3],fXYZStatMat[3];                      //

FbeHist *fHistXYZ[3];                                           //
bool fXYZGraphed[3];                                            //


protected:

long            _from,_howManyBytes;                            //
int             _dataNotYetRead;                                //
TArrayF         _data;                                          //

int             _withStatisticalErrorEstimate;                  //
long            _Sfrom,_ShowManyBytes;                          //
TArrayF         _Sdata;                                         //


fRemS           _Rem;                                           //



FbeUsrbin() { ; } // chains to TObject() constructor
FbeUsrbin(int ctr, int nr, char name[10], int type, int score, float arr[16], int from, int bytes);
virtual ~FbeUsrbin();

void addStatistics(int from, int bytes);


public:

inline const char *name() const { return (_theName); }
inline int type() const { return (_type); }
inline int score() const { return (_score); }

inline int getFlukaID() const { return (_flukaID); }
inline int indexof() const { return (_index); }
inline int rphi() const { return (_rphi); }
inline int rphiZ() const { return (_rphiz); }
inline int supported() const { return (_supported); }
inline int need2read() const { return (_dataNotYetRead); }
inline int rots() const { return (_rots); }

inline double min() const { return (_rawMin); }
inline double max() const { return (_rawMax); }

inline TArrayI bins() const { return (_bins); }
inline TArrayD limits() const { return (_limits); }
                                                                // ----------------
inline TArrayF raw3D() const { return(_data); }
inline TArrayF err3D() const { return(_Sdata); }
                                                                // ----------------

//
// --- push a new interface for filling the histogram which is to be displayed
//     we first have a method to fill a 2D array for the content and if present one for the
//     error, integrated over the 3rd coordinate - from now on we will call that 3rd coordinate
//     simply z even though in actuality it's cyclic through x/y/z
//     a second step will involve creating the actual 2D array/histogram to be displayed
//     the simple case is just the already created project2D, otherwise we try to rebin
//     in which case more summing is required -
//

void project2D(int view, int lz=0, int hz=0);
void rebin2D(int view, int lx=0, int hx=0, int Bx=1, int ly=0, int hy=0, int By=1);
void setRebin(int onoff) { _rebin= onoff; }
int  isRebinned() { return _rebin; }
void setWasRebinned() { _wasRebinned= 1; }

void fill2D(TArrayD cont, TArrayD err, int nx, int ny, FbeHist *fbeMH);
void triggerFill2D(int,int,int);

// ---

FbeHist *hist(int mode=0);
FbeHist *hist(int mode, double x0,double x1, double y0,double y1, double z0=-.9e99,double z1=.9e99, int=1,int=1);

FbeHist *GetHist(int view);

int WithStats() const { return _withStatisticalErrorEstimate; }


fRemS *getRem() { return &_Rem; }
int wasGraphed(int i) { return (fXYZGraphed[i]); }

private:

double dV(int xyz, int i);
double push2double(float f);
int edge(int view, double value, int upper=0) const;



ClassDef(FbeUsrbin,1)

}; // FbeUsrbin - class to hold a single usrbin binning


#endif
