
//
//    (feb'07 19-02:07) - version v.07a
//                                                                                 the FLEUR project
// -------------------------------------------------------------------------------------------------
//                                                                                anton.empl@cern.ch
// --- FLUKA user back-end project -Fbe-
//     base class to keep usrbin information (header/data) per file/run
//     read() is specific to the usrbin file structure in that it expects to read 4byte quantities
//     taken all built-in variables back to regular C/C++ types
//
//
//     13'mar  still missing some public interface methods to return information to a user
//             we will try and add more possible binary file formats to this class not just USRBIN
//
//     reading of geometry/input files is dealt with somewhere else (see try2open...)
//     processing of ASCII data files is not implemented
//


#ifndef __FBE_USRBIN_FILE_H
#define __FBE_USRBIN_FILE_H

#include <TObject.h>
#include <TObjArray.h>

#include <fcntl.h>

#include "FbeIO.h"
#include "FbeUsrbin.h"


#define DEBUG(arg)      {if (getenv("DEBUG")) arg;}

#define STATISTICAL_ERROR_AVAILABLE 14


class FbeUsrbinFile : public FbeIO, public TObject
{


private:

int            _withStatistics;                          //
char           _title[81],_date[33];                     // read from the usrbin file
int            _nrOfBinnings,_nrOfEvents;                //
float          _weightOfPrimary;                         //

TObjArray      _binning;                                 // per binning information collection


public:

FbeUsrbinFile(const char *aName);
virtual ~FbeUsrbinFile();

inline const char* filename() { return (_filename); }
inline const char* title() { return (_title); }
inline const char* date() { return (_date); }
inline const int nrOfEvents() { return (_nrOfEvents); }
inline const int nrOfBinnings() { return (_nrOfBinnings); }
inline const float weightOfPrimary() { return (_weightOfPrimary); }
inline const int WithStats() { return _withStatistics; }

inline const int status() { return (_status); }

FbeUsrbin *getABinning(int aBin);
void readData(FbeUsrbin* aBin);


ClassDef(FbeUsrbinFile,1)

}; // FbeUsrbinFile - class to inspect FLUKA usrbin binary data file

#endif
