
//
//    (feb'07 19-02:07) - version v.07a
//                                                                                 the FLEUR project
// -------------------------------------------------------------------------------------------------
//                                                                                anton.empl@cern.ch
//                                                                   (Katey Covin, Matt LeBourgeois)
// --- FLUKA user back-end project -Fbe-
//     this is the header for main() - set up the GUI and make use of the Fbe classes
//


#include <TApplication.h>
#include <TGResourcePool.h>
#include <TGFrame.h>
#include <TGLabel.h>
#include <TGButton.h>
#include <TGTextEntry.h>
#include <TGNumberEntry.h>
#include <TGMsgBox.h>
#include <TGMenu.h>
#include <TGComboBox.h>
#include "TGTab.h"
#include "TGFileDialog.h"
#include <TGTextEdit.h>
#include <TRootEmbeddedCanvas.h>
#include <TCanvas.h>
#include <TGButtonGroup.h>
#include <TColor.h>
#include <THistPainter.h>
#include <iostream>
#include <fstream>
#include <TGString.h>
#include <TVirtualX.h>

#include <TTimer.h>

#ifdef FOUND_XLIB
#include <TGX11.h>
#endif

#include <TGPicture.h>
#include <TImage.h>
#include <TSystem.h>
#include <TGraph.h>
#include <TList.h>

#include <KeySymbols.h>

#include <string.h>
#include <math.h>

#include "FlukaHelp.h"
#include "FbeUsrbinFile.h"
#include "FbeUsrbin.h"
#include "FbeIO.h"
#include "FbePlotgeom.h"
#include "TGBinEntry.h"
#include "aMessageBox.h"

#include "TEnv.h"
#include "TLatex.h"

#define DEBUG(arg)      {if (getenv("DEBUG")) arg;}


using namespace std;

#ifndef FlukaMainFrame_h
#define FlukaMainFrame_h

#include <signal.h>

extern "C" { pgmaxS *attach2geometry(char*); }
extern "C" { aptS *aPoint(float,float,float,int); }
extern "C" { void shutFlukaDown(); }
extern "C" { char *getInpDate(); }
extern "C" { char *getGeoDate(); }
extern "C" { char *getGeoFileName(); }

extern "C" { void georeg_(double*,double*,double*,int*,int*); }
extern "C" { void mypoint_(int*,double*,double*,double*); }
extern "C" { void getMaterialName(int*,char*); }
extern "C" { int rotprbin(int*); }


enum MessageTypes { MGETDATA,MGETGEOM,MGETINPUT,MGETFLUKAFILE,
                    MDATAFILE,MGEOMFILE,MINPUTFILE,MFLUKAFILE,
                    MBINNING, MXY,MYZ,MZX, MALLOWREBIN,

                    MXMIN,MYMIN,MZMIN,MXMAX,MYMAX,MZMAX,
                    MXBINSIZE,MYBINSIZE,MZBINSIZE,
                    MXBIN,MYBIN,MZBIN,

                    MHISTOIT2,
                    MSLICE,MPROJECTION,MXPROJ,MYPROJ,MPROJMIN,MPROJMAX,MPROJECTIT,
                    MRESET,MGEOMETRY,MPRESERVE,MFROMHISTO,MSETMANUAL,MSETMANUALFIELD,
                    MGRAPHTITLE, MXTITLE,MYTITLE,MZTITLE,
                    MSWAP,MZNORMALIZE,MZSCALE,MZSCALEVALUE,MZUPPER,MZLOWER,MLOGCHECK,
                    MENABLECOLOR,MPRINTOPTCHECK,MCOLORPERDEC,Mgrey,

                    MHISTOIT,Mredraw,
                    MFileOpen,MFileSave,MFileSaveAs, MFilePrintSetup,MFilePrint,MFileExit,MViewToolBar,
                    Mquit,Mnext,Mprev, MoData,MoInput, MDclose,MIclose,MGclose,
                    MHELPLICENSE,MHELPABOUT, MSAVE,MSAVEAS,
                    MMACRO,MROOT,MXML,MGEOLINES,MGEOCHECK, PICKUPRANGE,RESETRANGE,MAXRANGE,MGCOLOR,
                    GSWAP,GSHOW,GLINES,GMODE,MREGIONS, MzoomP,MzoomM,
                    Mpreserve,Mmanually,Mratio,Gpreserve,Gmanually,Gratio,
                    Mresize,Mffixed,Mfixed,Mfill,Mphysical,Mfphysical,

                    MGXL,MGXU,MGYL,MGYU,MGZL,MGZU, MGZD, MGBXY,MGBYZ,MGBZX
                  };


class FlukaMainFrame: public TGMainFrame
{

private:

TGLayoutHints  *fL0,*fL9,*fL00,*fL99,*fL98,*fL97,*fL96,*fL95,
               *fL1,*fL2,*fL3,*fL4,
               *fL5,*fL6,*fL7,*fL8,*fLL,*fLL2,*fLL3,*fLogoLayout,
               *fLL4,*fLL8,*fLL9,*fLL14,*fLL5,*fLL6,*fLL7;               // all/most of the layout hints

char whereAmI[256]; 
int _sepheight,_sepshape;

TGHorizontalFrame *fMainFrame,  *_hf1,*_holdCanvas;
TGVerticalFrame   *_vf1;
TGTextButton      *_resize,*_fill,*_fixed,*_physical;
TGCheckButton     *_ffixed,*_fphysical;

TColor 	          *matt,*grey,*dark,*light,*red;                         // colors (background, grey out) extra

TGHorizontalFrame *fTopBar;
TGMenuTitle       *fFileTitle,*fHelpTitle;
TGMenuBar         *fMenuBar;
TGLayoutHints     *fMenuBarLayout;
TGPopupMenu       *fMenuFile,*fMenuView,*fMenuHelp;
TGTextButton      *fQuit,*fNext,*fPrev, *foData,*foInput;
TGLabel *_alabel,*_blabel;

TGTab  *fMainTab;                                                        // main Tab, consits of 3(->4) tabs


// --- for now we leave this as part of the main class
//     we should move it into a independent class however later, no time now
//     these are our Tabs which were 'inlined' by Matt
//     nov'04 25-01:37
//

TGCompositeFrame   *fFrame4,*fTitleFrame4;
TGLabel            *fTitleGeo0,*fTitleGeo1,*fTitleGeo2,*fTitleGeo3,*fTitleGeo4,*fTitleGeo5;
TGCompositeFrame   *fFrame5,*fTitleFrame5;
TGLabel            *fTitleUsr0,*fTitleUsr1,*fTitleUsr2,*fTitleUsr3,*fTitleUsr4,*fTitleUsr5;
TGCompositeFrame   *fFrame6,*fTitleFrame6;
TGLabel            *fTitleInp0,*fTitleInp1,*fTitleInp2,*fTitleInp3,*fTitleInp3n,*fTitleInp4,*fTitleInp5,*fTitleInp6;

TGHorizontalFrame  *fGeoDecor0,*fGeoDecor1,*fGeoDecor2,*fGeoDecor3;
TGCompositeFrame   *fFrame7,*fTitleFrame7;
TGLabel            *fTitle7Geo0,*fTitle7Geo1,*fTitle7Geo2,*fTitle7Geo3,*fTitle7Geo4,*fTitle7Geo5, *fTitle7Geo7;

// ---(mar'06)---

TGCompositeFrame   *_frame;
TGHorizontalFrame  *_hRanges,*_hMinFrame,*_hMaxFrame,*_vzframe,*_h1box;
TGTextButton       *_redraw;
TGVerticalFrame    *_vaf,*_vbf,*fFrameVG;
TGLabel            *_LabelR,*_xMinL,*_xMaxL,*_yMinL,*_yMaxL,*_zMinL,*_zMaxL, *_hMinLabel,*_hMaxLabel,*_zLabel;
TGHButtonGroup     *_buttonGroup;
TGRadioButton      *_xyzB[3];
TGNumberEntry      *_gco[6],*_zd;

int                _gview,_nr[3];
double             _xyz[6],_z[3];   // view
char               *_xtitle,*_ytitle;

// ---


TGCompositeFrame   *fFrame1G,*fFrame2G,*fFrame3G;
TGHorizontalFrame  *fFrame4G,*fFrame5G,*fFrame5g,*fFrame6G,*fFrame7G,*fFrame8G,*fFrame8g,*fFrame9G;
TGLabel            *fGeoLabel20,*fGeoLabel23,*fGeoLabel24,*fGeoLabel25,*fGeoZoom,
                   *fGeoMaterials,*fGeoLmr[8],*fGeoLco[6];
TGNumberEntry      *fGeoLines,*fGeoWhere;
TGCheckButton      *_swap,*_show,*_lines,*_mode,*fGeoRegions,*fGeoColors;
TGTextButton       *fGeoButton1,*fGeoButton2,*fGeoButton3;

TGPictureButton    *fGeoZplus,*fGeoZminus;

//
// ---ae--


TGCompositeFrame   *fFrame1,*fTitleFrame1, *fTitleFrame1B;                // frames for the first tab
TGHorizontalFrame  *fFrame12,*fFrame11,*fFrame13,*fFrame14,*fFrameDecor0,
                   *fFakeFrame1,*fFakeFrame2,*fFakeFrame3,*fFakeFrame4,*fFrameDecor8;
TGLabel            *fTitle0,*fEmpty0,*fEmpty1,*fEmpty2,*fTitle1,*fTitle2,*fTitle3,*fTitle4,*fTitle5,
                   *fTitle1B,*fTitle2B;

TGTextEntry        *fTextData,*fTextGeom,*fTextInput,*fFlukaFile;
TGTextButton       *fData,*fGeom,*fInput,*fGetFlukaFile;

TGCompositeFrame   *fFrame2,*fTitleFrame2;                                // frames needed for second Tab
TGHorizontalFrame  *fFrameDecor5,*fFrameDecor4,*fFrameDecor7, *fFrame200,*fFrame22, *_yp;
TGVerticalFrame    *fFrame201;
TGCompositeFrame   *fFrame21;
TGVerticalLayout   *fBinLabels;
TGMatrixLayout     *fBinMatrix;
TGLabel            *fText2,*fText3,*fText4,*fText5,*fText0,*fText1,
                   *fBinningTitle,*fLabel,*fMinLabel,*fMaxLabel,*fBinSizeLabel,
                   *fPlane1,*fPlane2,*fPlane2g,*fPlane3;

TGComboBox         *fBinningFrame;                                        // combo box to select a binning

TGHButtonGroup     *fXYZButtonGroup;
TGRadioButton      *fXYButton,*fYZButton,*fZXButton;
TGLabel            *_xyLx,*_xyLy, *_yzLy,*_yzLz, *_xyLp,*_yzLp;
TGTextButton       *fAllowReBin;

// ---
// --- Matt's class

TGBinEntry *fXMin,*fXMax,*fXBin, *fYMin,*fYMax,*fYBin, *fZMin,*fZMax,*fZBin,   // hold viewing information for our histograms
           *fMix[6],*fBin[3], *fMin[3],*fMax[3];                               //
TGNumberEntry *fXBinSize,*fYBinSize,*fZBinSize,                                // binwidth
              *fSize[3];
// ---
// ---

TGTextButton *fHistoIt2;                                                  // button to update histogram
TGTextButton *fReset;                                                     // to reset histogram, actually reopens the data file
TGHorizontalFrame *fFrameDecor10,*fFrame23,*fFrame24,*fFrame25;           //
TGLabel *fProjLabel1,*fProjLabel2;
TGVButtonGroup *fSliceBG, *fViewBG;

TGRadioButton *fSlice,*fProjection,*fYProj,*fXProj;                       // use to Select a slice and a projection and hori/vert
TGBinEntry *fProjMin,*fProjMax;                                           // min/max integration range for projection
TGTextButton *fProjectIt;                                                 //
TGCompositeFrame *fFrame3,*fTitleFrame3;
TGLabel *fGraphIntro0,*fGraphIntro1,*fGraphIntro2,*fGraphIntro3,*fGraphIntro4,*fGraphIntro5;    // -ae-

TGHorizontalFrame *fFrame31,*fFrame32,*fFrame33,*fFrame34,*fFrame35,*fFrame39,*fFrame40,*fFrame37,*fFrame309,*fFrame38,
                  *fFrameDecor2,*fFrameDecor1,*fFrameDecor3,*fFrame3010,*fFrame3011, *fFrameDecor9;

TGCheckButton   *fGeometry;
TGVButtonGroup  *fViewButtons;                                            // button group for display of graphs
TGHButtonGroup  *_hxyzGroup;                                              // replicate/sync with the one in AddTab2
TGRadioButton   *_hxyz[3];
TGLabel         *fGraphTitleWords;
char            _AxisLabel[3][256];
TGTextEntry     *fGraphTitle,*fXTitle,*fYTitle,*fZTitle;
TGNumberEntry   *fZLower,*fZScaleValue,*fColorPerDec;                     // anchor, scale and colors-per-decade

TGLabel         *fXTitleWords,*fYTitleWords,*fZTitleWords,*fZRange,*fZFrom,*fZLabel,*fColorLabel;
TGRadioButton   *fZNormalize,*fZScale;
TGCheckButton   *fSwap,*fLogCheck,*fEnableColor,*fPrintOptCheck,*_grey;


TGHButtonGroup  *_aspectBG,*_aspectBGg;
TGCheckButton   *_preserve,*_manually,*_preserveG,*_manuallyG;
TGNumberEntry   *_ratio,*_ratioG;


TGTextButton    *fHistoIt;                                                // update histogram
TRootEmbeddedCanvas *c1;
Int_t fXYZID;                                                             // current plane, better view

Bool_t          fImportFlukaFile,fImportInput,fImportGeom,fImportData,
                fImportBinning,                                           // did we read a data file
                fImportFbeGeom,                                           // and/or has a geometry file been read
                fImportScan;                                              // is a plotgeom store file available and was it read

TArrayD         fMids;
TCanvas         *fProjCanvas;                                             // popup canvas for projection

double          fScaling;                                                 // current projection value ??
int             fGoingUp;                                                 // determine whether fProjection should be on or off
int             fIsSlice;                                                 // keeps track of whether we are slicing or projecting

TGFont *mf,*myFont,*greekFont;
TGGC mGC,myGC,greekGC;

int _six00;
int _requestBlack;

void remReset(int i);
void remSet(int i);
void remPush(int i);


// --- function, should read method, prototypes

void UpdateBinMatrix();
void SetPlane(int);
void WriteScript();
void ReadScript();
void MakeMenuBarFrame();
void AddTab1(TGCompositeFrame*, char*, char*, int, char*, char** );
void AddTab2(TGCompositeFrame *);
void AddTab3(TGCompositeFrame *);
void AddTab4(TGCompositeFrame *);
int GetCompression(int);
void SetProjArrays(int);

int try2openDataFile(const char*);
int try2openStoreFile(const char*);
int try2openInputFile(const char*);
int try2openScriptFile(const char*);
void OurStyle(TH2*,int mode=0);
int cutZturnedOn;

public:

TGPictureButton    *fCloseDB,*fCloseIB,*fCloseGB;



FlukaMainFrame(int ur, const TGWindow *p, UInt_t w, UInt_t h);
virtual ~FlukaMainFrame();


   FbeUsrbinFile 			*fmyUsrBins;		//Link to the backend with all the data
   FbePlotgeom				*fmyGeometry;     //Link to the backend geometry
   FbePlotgeom				*fmyGxyz[3];      //Link to the backend geometry
   void OpenFile();
   void UpdateHistogram(int skip=0);

   virtual void CloseWindow();
   void extraCloseWindow(int i) { if (i==MFileExit) CloseWindow(); };

   virtual Bool_t ProcessMessage(Long_t msg, Long_t parm1, Long_t);
   void ChangeRange();
   virtual Bool_t HandleKey(Event_t*);
   void ReflectRanges();
   void ChangeFocus();
   void CreateProjection();
   void Scale();
   void SliceSlot();
   void ProjectionSlot();

   void MyHExec(int vh=0);
   void MyVExec();
   void MySimpleExec();
   void dealColors();
   void zoomExec(int,int,int,TObject*);
   void zoomed(int m=0);

   void CloseHistSlot();
   void HistClicked(TGFrame*,Int_t);
   void AllowReBinSlot();
   void ChangePlane(int);  //MLML
   void ResetHist();       //MLML

   void FakeSlot() { UpdateHistogram(); }

   void pushTGBEtext(int);
   void entered(int);
   void entered() { entered(0); }
   void FocusChangeSlot(Event_t* ev) { if (ev->fType==kFocusIn) entered(1);
                                         else if (ev->fType==kFocusOut) entered(-1); }
   void SetBinMatrix();
   void doTabs(int);
   void doListBox();

   void swap();
   void updateGeometry(int mode=0);
   void drawGeometry();
   void pickRange(int mode=0);
   void focusChangeGeometry(int);
   void nothing() { ; }

   void resetCanvas(int mode=0);
   void resizeCanvas();
   void switchTab();
   void disconnectFile(int p=0);

   void setCanvasAspect(TCanvas*,double=1,int=0);
   void splash(TCanvas*);

   const TGPicture *openIcon(char*);

   int _userRoutines;
   FlukaMainFrame *getAPointer2us() { return this; }
   TGTab *getMainTab() { return fMainTab; }

   void setOurCursor(ECursor); 
   void hiddenRequest(int j) { _requestBlack= j; }

   ClassDef(FlukaMainFrame,0)

}; //Fluka GUI Main Class

#endif
