
//
//    (feb'07 19-02:07) - version v.07a
//                                                                                 the FLEUR project
// -------------------------------------------------------------------------------------------------
//                                                                            mlebourgeois@gmail.com
//                                                                                anton.empl@cern.ch
//

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// Changing the number in the widget will generate the event:           //
// kC_TEXTENTRY, kTE_TEXTCHANGED, widget id, 0.                         //
// Hitting the enter key will generate:                                 //
// kC_TEXTENTRY, kTE_ENTER, widget id, 0.                               //
// Hitting the tab key will generate:                                   //
// kC_TEXTENTRY, kTE_TAB, widget id, 0.                                 //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#include <KeySymbols.h>
#include <TTimer.h>
#include <TSystem.h>
#include <TGToolTip.h>
#include <Riostream.h>
#include <ctype.h>
#include "TGBinEntry.h"

// ClassImp(TGBinEntryField);
// ClassImp(TGBinEntry);

enum RealStyle2 {               // Style of real
   kRSInt = 0,                  // Integer
   kRSFrac = 1,                 // Fraction only
   kRSExpo = 2,                 // Exponent only
   kRSFracExpo = 3              // Fraction and Exponent
};

struct RealInfo2_t {
   RealStyle2 fStyle;           // Style of real
   Int_t fFracDigits;           // Number of fractional digits
   Int_t fFracBase;             // Base of fractional digits
   Int_t fIntNum;               // Integer number
   Int_t fFracNum;              // Fraction
   Int_t fExpoNum;              // Exponent
   Int_t fSign;                 // Sign
};

static Long_t IntStr2(const char *text)
{
   Long_t l = 0;
   Int_t Sign = 1;
   for (UInt_t i = 0; i < strlen(text); i++) {
      if (text[i] == '-') {
         Sign = -1;
      } else if ((isdigit(text[i])) && (l < 100000000)) {
         l = 10 * l + (text[i] - '0');
      }
   }
   return Sign * l;
}

//Stolen From TGNumberEntry.cxx
static Double_t StrToReal2(const char *text, RealInfo2_t & ri)
{
	char *s;
   char *Frac;
   char *Expo;
   char *Minus;
   char buf[256];

   if ((text == 0) || (strlen(text) == 0)) {
      ri.fStyle = kRSInt;
      ri.fIntNum = 0;
      ri.fSign = 1;
      return 0.0;
   }
   strncpy(buf, text, sizeof(buf) - 1);
   s = buf;
   Frac = strchr(s, '.');
   if (Frac == 0) {
      Frac = strchr(s, ',');
   }
   Expo = strchr(s, 'e');
   Minus = strchr(s, '-');
   if (Expo == 0) {
      Expo = strchr(s, 'E');
   }
   if ((Frac != 0) && (Expo != 0) && (Frac > Expo)) {
      Frac = 0;
   }
   if ((Minus != 0) && ((Expo == 0) || (Minus < Expo))) {
      ri.fSign = -1;
   } else {
      ri.fSign = 1;
   }
   if ((Frac == 0) && (Expo == 0)) {
      ri.fStyle = kRSInt;
   } else if (Frac == 0) {
      ri.fStyle = kRSExpo;
   } else if (Expo == 0) {
      ri.fStyle = kRSFrac;
   } else {
      ri.fStyle = kRSFracExpo;
   }
   if (Frac != 0) {
      *Frac = 0;
      Frac++;
   }
   if (Expo != 0) {
      *Expo = 0;
      Expo++;
   }
   ri.fIntNum = TMath::Abs(IntStr2(s));
   if (Expo != 0) {
      ri.fExpoNum = IntStr2(Expo);
   } else {
      ri.fExpoNum = 0;
   }
   if (ri.fExpoNum > 999) {
      ri.fExpoNum = 999;
   }
   if (ri.fExpoNum < -999) {
      ri.fExpoNum = -999;
   }
   ri.fFracDigits = 0;
   ri.fFracBase = 1;
   ri.fFracNum = 0;
   if (Frac != 0) {
      for (UInt_t i = 0; i < strlen(Frac); i++) {
         if (isdigit(Frac[i])) {
            if (ri.fFracNum < 100000000) {
               ri.fFracNum = 10 * ri.fFracNum + (Frac[i] - '0');
               ri.fFracDigits++;
               ri.fFracBase *= 10;
            }
         }
      }
   }
   if ((ri.fFracDigits == 0) && (ri.fStyle == kRSFrac)) {
      ri.fStyle = kRSInt;
   }
   if ((ri.fFracDigits == 0) && (ri.fStyle == kRSFracExpo)) {
      ri.fStyle = kRSExpo;
   }
   switch (ri.fStyle) {
   case kRSInt:
      return ri.fSign * ri.fIntNum;
   case kRSFrac:
      return ri.fSign *
          (ri.fIntNum + (Double_t) ri.fFracNum / ri.fFracBase);
   case kRSExpo: return ri.fSign * (ri.fIntNum * pow((double)10,ri.fExpoNum));
   case kRSFracExpo: return ri.fSign * (ri.fIntNum + (Double_t) ri.fFracNum / ri.fFracBase) * pow((double)10,ri.fExpoNum);
   }
   return 0;
}

//Stolen From TGNumberEntry.cxx
static Bool_t IsGoodChar2(char c, TGNumberFormat::EStyle style,
                         TGNumberFormat::EAttribute attr)
{
   if (isdigit(c)) {
      return kTRUE;
   }
   if (isxdigit(c) && (style == TGNumberFormat::kNESHex)) {
      return kTRUE;
   }
   if ((c == '-') && (style == TGNumberFormat::kNESInteger) &&
       (attr == TGNumberFormat::kNEAAnyNumber)) {
      return kTRUE;
   }
   if ((c == '-') &&
       ((style == TGNumberFormat::kNESRealOne) ||
        (style == TGNumberFormat::kNESRealTwo) ||
        (style == TGNumberFormat::kNESRealThree) ||
        (style == TGNumberFormat::kNESRealFour) ||
        (style == TGNumberFormat::kNESReal) ||
        (style == TGNumberFormat::kNESDegree) ||
        (style == TGNumberFormat::kNESMinSec)) &&
       (attr == TGNumberFormat::kNEAAnyNumber)) {
      return kTRUE;
   }
   if ((c == '-') && (style == TGNumberFormat::kNESReal)) {
      return kTRUE;
   }
   if (((c == '.') || (c == ',')) &&
       ((style == TGNumberFormat::kNESRealOne) ||
        (style == TGNumberFormat::kNESRealTwo) ||
        (style == TGNumberFormat::kNESRealThree) ||
        (style == TGNumberFormat::kNESRealFour) ||
        (style == TGNumberFormat::kNESReal) ||
        (style == TGNumberFormat::kNESDegree) ||
        (style == TGNumberFormat::kNESMinSec) ||
        (style == TGNumberFormat::kNESHourMin) ||
        (style == TGNumberFormat::kNESHourMinSec) ||
        (style == TGNumberFormat::kNESDayMYear) ||
        (style == TGNumberFormat::kNESMDayYear))) {
      return kTRUE;
   }
   if ((c == ':') &&
       ((style == TGNumberFormat::kNESDegree) ||
        (style == TGNumberFormat::kNESMinSec) ||
        (style == TGNumberFormat::kNESHourMin) ||
        (style == TGNumberFormat::kNESHourMinSec) ||
        (style == TGNumberFormat::kNESDayMYear) ||
        (style == TGNumberFormat::kNESMDayYear))) {
      return kTRUE;
   }
   if ((c == '/') &&
       ((style == TGNumberFormat::kNESDayMYear) ||
        (style == TGNumberFormat::kNESMDayYear))) {
      return kTRUE;
   }
   if (((c == 'e') || (c == 'E')) && (style == TGNumberFormat::kNESReal)) {
      return kTRUE;
   }
   return kFALSE;
}




//______________________________________________________________________________
TGBinEntryField::TGBinEntryField(const TGWindow * parent,
                                      TArrayD vals, Int_t id)
   : TGTextEntry(parent, "", id), fNumStyle(kNESReal), fNumAttr(kNEAAnyNumber), fI(0)
{

	Float_t temp = 0.0;
	fValues.Set(vals.GetSize(),vals.GetArray());
	for(int i=0;i<fValues.GetSize()-1;++i)
		for(int j=i+1;j<fValues.GetSize();++j)
			if(fValues[i]>fValues[j])
			{
				temp = fValues[i];
				fValues[i] = fValues[j];
				fValues[j] = temp;
			}


	// Constructs a number entry field
   SetAlignment(kTextRight);
   SetNumber(0);
}


//
// --- feb'06 25-04:30
//     these are Matt's and we need to watch out since the FORMAT is very fixed here
//     much too static and limited - needs work
//

void TGBinEntryField::SetNumber(Int_t i)
{
   if (i>(fValues.GetSize()-1)) fI= fValues.GetSize()-1;
     else if (i<0) fI= 0;
           else fI= i;

   char text[256];
   strcpy(text,"");
   sprintf(text, "%7.3g", (TMath::Abs(fValues[fI])<=0.0001) ? 0:fValues[fI]);
   SetText(text);

   Emit("SetNumber(Int_t)",i);
}


void TGBinEntryField::SetNumber(Double_t val)
{
   int i,closest=0;
   double first=0,last=TMath::Abs(fValues[0]-val);
   double current=last;

   for (i=1;i<fValues.GetSize();++i) {
      first= TMath::Abs(fValues[i]-val);
      if (first<=current) { closest= i; current= first; }
      if (first>last) break;
        else last= first;
      }

   fI= closest;

   char text[256];
   strcpy(text,"");
   sprintf(text, "%7.3g", (TMath::Abs(fValues[fI])<=0.0001) ? 0: fValues[fI]);
   SetText(text);
}


void TGBinEntryField::SetNumber()
{
   char text[256];
   RealInfo2_t ri;
   strcpy(text, GetText());
   SetNumber(StrToReal2(text,ri));
}


Double_t TGBinEntryField::GetNumber()   { return fValues[fI]; }

// ---


//______________________________________________________________________________
Int_t TGBinEntryField::GetCharWidth(const char *text) const
{
   // Get the text width in pixels
   return gVirtualX->TextWidth(fFontStruct, text, strlen(text));
}

//______________________________________________________________________________
void TGBinEntryField::SetState(Bool_t state)
{
   TGTextEntry::SetState(state);
}

//______________________________________________________________________________
Bool_t TGBinEntryField::HandleKey(Event_t * event)
{
   // Handle keys

   if (!IsEnabled()) {
      return TGTextEntry::HandleKey(event);
   }

   Int_t n;
   char tmp[10];
   UInt_t keysym;
   gVirtualX->LookupString(event, tmp, sizeof(tmp), keysym);
   n = strlen(tmp);

   // intercept up key
   if ((EKeySym) keysym == kKey_Up) {
      // shift-cntrl-up I dont care about shift-cntrl
      if ((event->fState & kKeyShiftMask) &&
          (event->fState & kKeyControlMask)) {
         //IncreaseNumber(kNSSHuge, 1, logstep);
      }
      // cntrl-up
      else if (event->fState & kKeyControlMask) {
         SetNumber(fValues.GetSize()-1);
      }
      // shift-up I dont care about shift
      else if (event->fState & kKeyShiftMask) {
         //IncreaseNumber(kNSSMedium, 1, logstep);
      }
      // up
      else if( fI< (fValues.GetSize()-1) ){
         SetNumber(fI+1);
      }
      return kTRUE;
   }
   // intercept down key
   else if ((EKeySym) keysym == kKey_Down) {
      // shift-cntrl-down I dont care
      if ((event->fState & kKeyShiftMask) &&
          (event->fState & kKeyControlMask)) {
         //IncreaseNumber(kNSSHuge, -1, logstep);
      }
      // cntrl-down
      else if (event->fState & kKeyControlMask) {
         SetNumber(0);
      }
      // shift-down I dont care
      else if (event->fState & kKeyShiftMask) {
         //IncreaseNumber(kNSSMedium, -1, logstep);
      }
      // down
      else if( fI > 0 ){
         SetNumber(fI-1);
      }
      return kTRUE;
   }
   // intercept printable characters
   else if (n && (keysym < 127) && (keysym >= 32) &&
            ((EKeySym) keysym != kKey_Delete) &&
            ((EKeySym) keysym != kKey_Backspace) &&
            ((event->fState & kKeyControlMask) == 0)) {
      if (IsGoodChar2(tmp[0], fNumStyle, fNumAttr)) {
         return TGTextEntry::HandleKey(event);
      } else {
         return kTRUE;
      }
   }
   // otherwise use default behaviour
   else {
//    printf(" ==/==> aerger %i\n",keysym);
      return TGTextEntry::HandleKey(event);
   }
}

//______________________________________________________________________________
Bool_t TGBinEntryField::HandleFocusChange(Event_t * event)
{
   // Handle focus change

   if (IsEnabled() && (event->fCode == kNotifyNormal) &&
       (event->fState != kNotifyPointer) && (event->fType == kFocusOut)) {
      // make sure we have a valid number by increasing it by 0
		SetNumber();
   }
	Emit("HandleFocusChange(Event_t*)",(Long_t)event);
   return TGTextEntry::HandleFocusChange(event);
}

//______________________________________________________________________________
void TGBinEntryField::TextChanged(const char *text)
{
   // Text has changed message
   TGTextEntry::TextChanged(text);
}


//______________________________________________________________________________
void TGBinEntryField::Layout()
{
   // Layout
   if (GetAlignment() == kTextRight) {
      End(kFALSE);
   } else {
      Home(kFALSE);
   }
}


//______________________________________________________________________________
void TGBinEntryField::SetArray(const TArrayD &array)
{
	fValues.Set(array.GetSize(),array.GetArray());
}

//______________________________________________________________________________
void TGBinEntryField::SetArray(Double_t *array, const Int_t &i)
{
	fValues.Set(i,array);
}

void TGBinEntryField::SetElement(const Double_t & val, const Int_t & i)
{
	fValues[i]=val;
}

void TGBinEntryField::SetLength(const Int_t &i)
{
	fValues.Set(i);
}

Int_t TGBinEntryField::GetLength()
{
	return fValues.GetSize();
}


// -ae-
//////////////////////////////////////////////////////////////////////////
// TRepeatTimer                                                         //
// timer for numeric control box buttons                                //
//////////////////////////////////////////////////////////////////////////

class TGRepeatFireButton;

//______________________________________________________________________________
class TRepeatTimer : public TTimer {
private:
   TGRepeatFireButton *fButton;	// Fire button

 public:
   TRepeatTimer(TGRepeatFireButton * button, Long_t ms)
    : TTimer(ms, kTRUE), fButton(button) { }
   virtual Bool_t Notify();
};



//////////////////////////////////////////////////////////////////////////
// TRepeatFireButton                                                    //
// Picture button which fires repeatly as long as the button is pressed //
//////////////////////////////////////////////////////////////////////////

//______________________________________________________________________________
class TGRepeatFireButton : public TGPictureButton {
protected:
   TRepeatTimer             *fTimer;
   Int_t                     fIgnoreNextFire;
   TGNumberFormat::EStepSize fStep;
   Bool_t                    fStepLog;
   Bool_t                    fDoLogStep;

public:
   TGRepeatFireButton(const TGWindow *p, const TGPicture *pic,
                      Int_t id, Bool_t logstep)
    : TGPictureButton(p, pic, id), fTimer(0), fIgnoreNextFire(0),
       fStep(TGNumberFormat::kNSSSmall), fStepLog(logstep) { }
   virtual ~TGRepeatFireButton() { delete fTimer; }

   virtual Bool_t HandleButton(Event_t *event);
   void FireButton();
   virtual void SetLogStep(Bool_t on = kTRUE) { fStepLog = on; }
};

//______________________________________________________________________________
Bool_t TGRepeatFireButton::HandleButton(Event_t * event)
{
   //

   const Int_t t0 = 200;
   if (fTip)
      fTip->Hide();

   if (fState == kButtonDisabled)
      return kTRUE;

   if (event->fType == kButtonPress) {
      // Get log step / alt key
      fDoLogStep = fStepLog;
      if (event->fState & kKeyMod1Mask)
         fDoLogStep = !fDoLogStep;
      if ((event->fState & kKeyShiftMask) &&
          (event->fState & kKeyControlMask)) {
         fStep = TGNumberFormat::kNSSHuge;
      } else if (event->fState & kKeyControlMask) {
         fStep = TGNumberFormat::kNSSLarge;
      } else if (event->fState & kKeyShiftMask) {
         fStep = TGNumberFormat::kNSSMedium;
      } else {
         fStep = TGNumberFormat::kNSSSmall;
      }
      SetState(kButtonDown);
      fIgnoreNextFire = 0;
      FireButton();
      fIgnoreNextFire = 2;

      if (fTimer == 0) {
         fTimer = new TRepeatTimer(this, t0);
      }
      fTimer->Reset();
      gSystem->AddTimer(fTimer);
   } else {
      SetState(kButtonUp);
      if (fTimer != 0) {
         fTimer->Remove();
         fTimer->SetTime(t0);
      }
   }

   return kTRUE;
}

//______________________________________________________________________________
void TGRepeatFireButton::FireButton()
{
   if (fIgnoreNextFire <= 0) {
      SendMessage(fMsgWindow, MK_MSG(kC_COMMAND, kCM_BUTTON),
                  fWidgetId, (Long_t) fStep + (fDoLogStep ? 100 : 0));
   } else {
      fIgnoreNextFire--;
   }
}

//______________________________________________________________________________
Bool_t TRepeatTimer::Notify()
{
   fButton->FireButton();
   Reset();
   if ((long)fTime>20) fTime -= 10;
   return kFALSE;
}

// -ae-
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TGBinEntryLayout                                                     //
// Layout manager for specific number entry widget                      //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

//______________________________________________________________________________
class TGBinEntryLayout : public TGLayoutManager {
protected:
   TGBinEntry *fBox;        // pointer to numeric control box

public:
   TGBinEntryLayout(TGBinEntry *box) { fBox = box; }
   virtual void Layout();
   virtual TGDimension GetDefaultSize() const;
};

//______________________________________________________________________________
void TGBinEntryLayout::Layout()
{
   if (fBox == 0) {
      return;
   }
   UInt_t w = fBox->GetWidth();
   UInt_t h = fBox->GetHeight();
   UInt_t upw = 2 * h / 3;
   UInt_t uph = h / 2;
   Int_t upx = (w > h) ? (Int_t) w - (Int_t) upw : -1000;
   Int_t upy = 0;
   Int_t downx = (w > h) ? (Int_t) w - (Int_t) upw : -1000;
   Int_t downy = h / 2;
   UInt_t downw = upw;
   UInt_t downh = h - downy;
   UInt_t numw = (w > h) ? w - upw : w;
   UInt_t numh = h;
   if (fBox->GetNumberEntry())
      fBox->GetNumberEntry()->MoveResize(0, 0, numw, numh);
   if (fBox->GetButtonUp())
      fBox->GetButtonUp()->MoveResize(upx, upy, upw, uph);
   if (fBox->GetButtonDown())
      fBox->GetButtonDown()->MoveResize(downx, downy, downw, downh);
}

//______________________________________________________________________________
TGDimension TGBinEntryLayout::GetDefaultSize() const
{
   return fBox->GetSize();
}

//______________________________________________________________________________
TGBinEntry::TGBinEntry(const TGWindow *parent, const TArrayD &val,
                 Int_t wdigits, Int_t id)
 : TGCompositeFrame(parent,10*wdigits,25), fButtonToNum(kTRUE)
{
	fWidgetId  = id;
   fMsgWindow = parent;
   fPicUp = fClient->GetPicture("arrow_up.xpm");
   if (!fPicUp)
      Error("TGBinEntry", "arrow_up.xpm not found");
   fPicDown = fClient->GetPicture("arrow_down.xpm");
   if (!fPicDown)
      Error("TGBinEntry", "arrow_down.xpm not found");

   // create gui elements
   fNumericEntry = new TGBinEntryField(this, val, id);
   fNumericEntry->Associate(this);
   AddFrame(fNumericEntry, 0);
   fButtonUp = new TGRepeatFireButton(this, fPicUp, 1,0);
   fButtonUp->Associate(this);
   AddFrame(fButtonUp, 0);
   fButtonDown = new TGRepeatFireButton(this, fPicDown, 2,0);
   fButtonDown->Associate(this);
   AddFrame(fButtonDown, 0);

   // resize
   Int_t h = fNumericEntry->GetDefaultHeight();
   Int_t charw = fNumericEntry->GetCharWidth("0123456789");
   Int_t w = charw * TMath::Abs(wdigits) / 10 + 8 + 2 * h / 3;
   SetLayoutManager(new TGBinEntryLayout(this));
   Resize(w, h);
}

//______________________________________________________________________________
TGBinEntry::TGBinEntry(const TGWindow *parent,
                             Double_t val[], Int_t numvals, Int_t wdigits, Int_t id)
 : TGCompositeFrame(parent, 10 * wdigits, 25), fButtonToNum(kTRUE)
{
   // Constructs a numeric entry widget

   fWidgetId  = id;
   fMsgWindow = parent;
   fPicUp = fClient->GetPicture("arrow_up.xpm");
   if (!fPicUp)
      Error("TGBinEntry", "arrow_up.xpm not found");
   fPicDown = fClient->GetPicture("arrow_down.xpm");
   if (!fPicDown)
      Error("TGBinEntry", "arrow_down.xpm not found");

	TArrayD temp(numvals,val);
   // create gui elements
   fNumericEntry = new TGBinEntryField(this, temp, id);
   fNumericEntry->Associate(this);
   AddFrame(fNumericEntry, 0);
   fButtonUp = new TGRepeatFireButton(this, fPicUp, 1,0);
   fButtonUp->Associate(this);
   AddFrame(fButtonUp, 0);
   fButtonDown = new TGRepeatFireButton(this, fPicDown, 2,0);
   fButtonDown->Associate(this);
   AddFrame(fButtonDown, 0);

   // resize
   Int_t h = fNumericEntry->GetDefaultHeight();
   Int_t charw = fNumericEntry->GetCharWidth("0123456789");
   Int_t w = charw * TMath::Abs(wdigits) / 10 + 8 + 2 * h / 3;
   SetLayoutManager(new TGBinEntryLayout(this));
   Resize(w, h);
}

//______________________________________________________________________________
TGBinEntry::~TGBinEntry()
{
   // Destructs a numeric entry widget
   delete fButtonUp;
   delete fButtonDown;
   delete fNumericEntry;
}

//______________________________________________________________________________
void TGBinEntry::Associate(const TGWindow *w)
{
   // Make w the window that will receive the generated messages.
   TGWidget::Associate(w);
}

//______________________________________________________________________________
void TGBinEntry::SetState(Bool_t enable)
{
   // Set the active state
   if (enable) {
      fButtonUp->SetState(kButtonUp);
      fButtonDown->SetState(kButtonUp);
      fNumericEntry->SetState(kTRUE);
   } else {
      fButtonUp->SetState(kButtonDisabled);
      fButtonDown->SetState(kButtonDisabled);
      fNumericEntry->SetState(kFALSE);
   }
}


void TGBinEntry::SetButtonToNum(Bool_t state)   { fButtonToNum = state; }


//______________________________________________________________________________
Bool_t TGBinEntry::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2)
{
	switch (GET_MSG(msg))
	{
   	case kC_COMMAND:
      {
         if(GET_SUBMSG(msg) == kCM_BUTTON)
			{	if(fButtonToNum) switch(parm1)
				{
					case 1: 	fNumericEntry->IncreaseNumber();
								ValueSet(10000 * (parm1 - 1) + parm2);
								break;
					case 2:	fNumericEntry->IncreaseNumber(1);
								ValueSet(10000 * (parm1 - 1) + parm2);
								break;
				}
				else
				{
					SendMessage(fMsgWindow, msg, fWidgetId, parm1);
               ValueChanged(10000 * (parm1 - 1) + parm2);
				}  }
		} break;
		case kC_TEXTENTRY: switch(GET_SUBMSG(msg))
		{
			case kTE_TAB:
			case kTE_ENTER: fNumericEntry->SetNumber();
// 						SendMessage(fMsgWindow,msg,fWidgetId,parm2);
						ValueChanged(10000 * (parm1 - 1) + parm2);
				break;
		} break;
   }
   return kTRUE;
}



//
// ---
//

void TGBinEntry::ValueChanged(Long_t val)   { Emit("ValueChanged(Long_t)", val); }
void TGBinEntry::ValueSet(Long_t val)   { Emit("ValueSet(Long_t)", val); }
void TGBinEntry::SetArray(const TArrayD &array)   { fNumericEntry->SetArray(array); }
void TGBinEntry::SetArray(Double_t *array, const Int_t &i)   { fNumericEntry->SetArray(array,i); }
void TGBinEntry::SetElement(const Double_t & val, const Int_t & i)   { fNumericEntry->SetElement(val,i); }
void TGBinEntry::SetLength(const Int_t &i)   { fNumericEntry->SetLength(i); }
 int TGBinEntry::GetLength()   { return fNumericEntry->GetLength(); }


