
//
//    (feb'07 19-02:07) - version v.07a
//
// -------------------------------------------------------------------------------------------------
//                                                                       the FLEUR project
//                                                                       author: anton.empl@cern.ch
//                                                                              mlebourgeois@uh.edu
// -- FLUKA user back-end project -Fbe-
//
//    this is our main() routine - it sets up the GUI and makes use of the Fbe classes
//

#ifndef TGBinEntry_h
#define TGBinEntry_h

//////////////////////////////////////////////////////////////////////////
//                                                                      //
//                                                                      //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TGFrame
#include <TGFrame.h>
#endif
#ifndef ROOT_TGTextEntry
#include <TGTextEntry.h>
#endif
#ifndef ROOT_TGButton
#include <TGButton.h>
#endif
#ifndef ROOT_TArrayD
#include <TArrayD.h>
#endif
#ifndef ROOT_TGNumberEntry
#include <TGNumberEntry.h>
#endif

#include <math.h>


class TGBinEntryField : public TGTextEntry, public TGNumberFormat {

protected:

   EStyle        fNumStyle;          // Number style
   EAttribute    fNumAttr;           // Number attribute
   Int_t         fI;                 // Current index
   TArrayD       fValues;            // Array of possible values;

public:

   TGBinEntryField(const TGWindow *parent, TArrayD vals, Int_t id=-1);
   virtual void SetNumber(Int_t i);
   virtual void SetNumber(Double_t val);

   void  IncreaseNumber(Int_t i=0) { if (i) {if(fI>0) SetNumber(fI-1);}
                                       else {if(fI<(fValues.GetSize()-1)) SetNumber(fI+1);} }

   void ChangeIndex(int i)   { IncreaseNumber(i-1); }

   Int_t GetIndex()   { return fI; }
   virtual Double_t GetNumber();
   void SetNumber();
   virtual Int_t GetCharWidth(const char* text = "0") const;
   virtual void SetState(Bool_t state);

   virtual EStyle GetNumStyle() const   { return fNumStyle; }
   virtual EAttribute GetNumAttr() const   { return fNumAttr; }

   virtual Bool_t HandleKey(Event_t* event);
   virtual Bool_t HandleFocusChange (Event_t* event);  //*SIGNAL*
   virtual void TextChanged(const char *text = 0);
   virtual void Layout();

   void SetArray(const TArrayD &array);
   void SetArray(Double_t *array, const Int_t &i);
   void SetElement(const Double_t & val, const Int_t & i);
   void SetLength(const Int_t &i);
   Int_t GetLength();
   TArrayD GetArray() { return fValues; }

   ClassDef(TGBinEntryField,1)
};



class TGBinEntry : public TGCompositeFrame, public TGWidget,
   public TGNumberFormat {

private:
   const TGPicture  *fPicUp;        // Up arrow
   const TGPicture  *fPicDown;      // Down arrow

protected:
   TGBinEntryField *fNumericEntry;  // Number text entry field
   TGButton        *fButtonUp;      // Button for increasing value
   TGButton        *fButtonDown;    // Button for decreasing value
   Bool_t          fButtonToNum;

public:
   TGBinEntry(const TGWindow *parent, const TArrayD &val, Int_t digitwidth=5, Int_t id=-1);
   TGBinEntry(const TGWindow *parent, Double_t val[], Int_t numvals, Int_t digitwidth=5, Int_t id=-1);

   virtual ~TGBinEntry();
   void SetArray(const TArrayD &);
   void SetArray(Double_t *, const Int_t &);
   void SetElement(const Double_t &, const Int_t &);
   void SetLength(const Int_t &);
   Int_t GetLength();

   Double_t GetElement(const Int_t &i)   const {return fNumericEntry->GetNumber(); }
   virtual void SetNumber(Double_t val)   { fNumericEntry->SetNumber(val); }
   virtual void SetState(Bool_t enable = kTRUE);
   virtual Double_t GetNumber()   const { return fNumericEntry->GetNumber(); }

   virtual EStyle GetNumStyle()   const { return fNumericEntry->GetNumStyle(); }
   virtual EAttribute GetNumAttr()   const { return fNumericEntry->GetNumAttr(); }

   virtual void Associate(const TGWindow *w);
   virtual Bool_t ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2);
   virtual void ValueChanged(Long_t val);    //*SIGNAL*
   virtual void ValueSet(Long_t val);    //*SIGNAL*

   void SetButtonToNum(Bool_t);
   TGBinEntryField *GetNumberEntry() const { return fNumericEntry; }

   TGButton *GetButtonUp() const { return fButtonUp; }
   TGButton *GetButtonDown() const { return fButtonDown; }

   Int_t GetIndex()   { return fNumericEntry->GetIndex(); }
   void ChangeIndex(int i)   { fNumericEntry->ChangeIndex(i); }
   void SetIndex(int i)   { fNumericEntry->SetNumber(i); }
   TArrayD GetArray()   { return fNumericEntry->GetArray(); }

   ClassDef(TGBinEntry,1)  // Entry field widget for several numeric formats
};

#endif
