##
## --- sep'06 14-16:14
##
##     will transform the original csh script into python to ease this
##     procedure and make it easier to maintain and cleaner in coding
##     it's run by a bash configure wrapper to find python first
##
##     check if we can find libg2c.a which is required since we make use of gcc/g++/g77
##     (gfortran not working yet for FLUKA)
##     determine root/fluka versions plus fluka compile options
##     add a list of optional fluka user routines
##     all given back as a dictionary
##
##     this works under linux
##
## --- sep'06 23'13:07
##
##     tried to make search for libg2c more robust, ldd output format?
##     also hook into finding libX11
##


import sys,getopt,os,re
import time,stat

pwd= os.environ['PWD']
fromSRC= '../'
flupro= None
xlib= ''

# ----------------------- just to ease use of system calls ---
def syscall(cmd=None):
 a,b,c= os.popen3(cmd)
 out= b.read()
 err= c.read()
 a.close()
 b.close()
 c.close()
 return out,err
# --------------


## --- sep'06 25-11:23
##     the nice way to parse our commandline options does not work for python 2.2
##     lxplus sits on python 2.2

try:
 getopt.gnu_getopt([],"")
 ourGetopt= getopt.gnu_getopt
except: ourGetopt= getopt.getopt


##
## --- process command line arguments

def usage(cmd):
   print ' --- usage: ',cmd,' [options]'
   print ''
   print '     options:'
   print '       -h, --help]                          show this help message and exit'
   print '       -l path, --libg2c path               specify libg2c (like /usr/lib/libg2c.so.0)'
   print '       -p path, --prefix path               place to install flukaGUI to, default  $FLUPRO/flutil'
   print '      --------------------------- -p/--prefix --- NOT IMPLEMENTED YET ----'
   print '       -u filename(s), --user filename(s)   list of user routines required for FLUKA geometry'
   print ''


def parse(argv):
   global flupro
   try: opts,args= ourGetopt(argv[2:], "hu:p:l:", ["help","user=","prefix=","libg2c="])
   except getopt.GetoptError:
         usage(argv[0])
         sys.exit(2)

   for opt,arg in opts:
      if opt in ('-h', '--help'):
        usage(argv[0])
        sys.exit()

   prefix= os.path.join(flupro,'flutil')
   userfiles= None
   lpath= None
   xpath= None

   for opt,arg in opts:
      if opt in ('-l','--libg2c'): lpath= arg
      if opt in ('-p','--prefix'): prefix= arg
      if opt in ('-u','--user'): userfiles= [arg,]

## ---
   if args: 
     if not userfiles: sys.exit(' ---> trouble with user files, did you give -u/--user?, exit')
     userfiles+= args

   dates= {}
   if userfiles:
     for afile in userfiles:
        if not os.path.isfile(afile): sys.exit(' ---> trouble with '+afile+' - can not find it, exit')
        else: dates[afile.split('/')[-1].split('.')[0]]= time.ctime(os.stat(afile)[stat.ST_MTIME])
# -
   for k in dates.keys():
      mon= dates[k].split()[1]
      day= dates[k].split()[2]
      year= dates[k].split()[-1]
      dates[k]= '('+day+'/'+mon.lower()+'/'+year[2:]+')'

   return prefix,userfiles,dates,lpath,xpath


##
## --- prepare the list of FLUKA user routines into makefile style for us
##     check here if we can find the files (at least for right now)

def format4make(files):
   fortran= ''
   objects= ''

   for afile in files:
      if afile.endswith(r'.f'):
        objects+= ' '+afile.replace('.f','.o')
        if afile.startswith('/'): fortran+= ' '+afile
        else: fortran+= ' '+os.path.join(fromSRC,afile)
      else:
            if afile.endswith(r'.o') or afile.endswith(r'.a'):
              if afile.startswith('/'): objects+= ' '+afile
              else:
                    if afile.startswith('src'): objects+= afile.replace('src/',' ')
                    else: objects+= ' '+os.path.join(fromSRC,afile)
            else:
                  print ' ---> trouble with supplied user files'
                  print '     only .o .f and .a are supported, exit'
                  sys.exit(2)

   return objects,fortran


##
## --- this is really where it's at
##     determine  libg2c/root/fluka  conditions on this system

def determineStuff(mode=None):
   global flupro,xlib
   dict= {}

   print '     attempt to configure flukaGUI, v07'
   user= None
   fortranfiles= None

## --- environment
## -------------------------------------------------------------------------------------------
   try: flupro= os.environ['FLUPRO']
   except:
           print ' ---> $FLUPRO not defined, exit'
           sys.exit(1)

   try: root= os.environ['ROOTSYS']
   except:
           print ' ---> $ROOTSYS not defined, exit'
           sys.exit(1)


## --- command line arguments
## -------------------------------------------------------------------------------------------
   prefix,userfiles,dates,lpath,xpath= parse(sys.argv[1:])
   dict['dates']= dates
   dict['prefix']= prefix


## --- turn simple list of user routines makefile style
## -------------------------------------------------------------------------------------------
   if userfiles: user,fortranfiles= format4make(userfiles)
   dict['user']= user
   dict['fortran']= fortranfiles

   if dict['user'] and mode: return dict



## --- libX11.a
##     check for it's proper install, if not just disable the tiny feature we use directly
## -------------------------------------------------------------------------------------------
   res,err= syscall('which gcc')
   gcc= res.strip()
   if gcc.endswith('gcc'): pass
   else: sys.exit(' ---> trouble, could not find gcc which is required for build, exit')

   prog= '#include <stdlib.h>\n#include <stdio.h>\n#include <X11/Xlib.h>\n'
   prog+= r'int main() { Display *d= XOpenDisplay(\" \"); return 0; }'+'\n'

   os.system('echo "'+prog+'" > __t.c')
   res,err= syscall('gcc -m32 -o __t __t.c -lX11')

   if err=="": xlib= '-lX11'
   else:
         res,err= syscall('gcc -m32 -o __t __t.c -I/usr/X11R6 -L/usr/X11R6/lib -lX11')
         if err=="": xlib= '-L/usr/X11R6/lib -lX11'

   os.system('rm -f __t __t.c')


## --- libg2c.a
## -------------------------------------------------------------------------------------------
   res,err= syscall('which g77')
   g77= res.strip()
   if g77.endswith('g77'): pass
   else:
    res,err= syscall('which f77')
    if g77.endswith('f77'): pass
    else:
          sys.exit(' ---> trouble, could not find g77 which is required for build, exit')

   libg2c= None

   if lpath:
     res,err= syscall('ls '+lpath)
     if lpath.strip()==res.strip(): libg2c= lpath
     else: sys.exit(' ---> trouble, specified libg2c not found, exit')

   else:
    os.system('echo "      i=7\n      end" > __t.f;'+ g77 +' -m32 -o __t __t.f >& /dev/null')
    res,err= syscall('ldd __t; rm -f __t __t.f')
    for item in res.split('\n'):
       if item.strip().startswith('libg2c'):
         items= item.strip().split()
         if items[-1].startswith('(') and items[-1].endswith(')'):
           libg2c= items[-2]
         else: print '    > ldd output seems to not show a dynamic libg2c'
    if not libg2c:
      print ' ---> trouble, could not find required libg2c, exit'
      print '    > try to supply the correct path to libg2c with -l/--libg2c'
      sys.exit(2)

   dict['libg2c']= libg2c


## --- root
## -------------------------------------------------------------------------------------------
   kludge,err= syscall('$ROOTSYS/bin/root-config --version')
   version= kludge.strip()
   xx= ' '
   if xlib: xx= '(X11)'
   print '      - root version ',version, xx

   if version and version[-1].isalpha():
     version= version[:-1]
     print '        ---> warning root version contains subversion qualifier'
     print '             this might cause problems, maybe upgrade root?'

   if not os.path.isdir('patch/root_v'+version.replace('/','.')):
     version= 'generic'
     print '        ---> the installed version of root does not correspond to one of the'
     print '             supported versions - a generic application will be built'
     print '             ISSUES: a few esthetical and a root canvas keyboard focus problem'
     print '                     "swap" will not work for USRBIN data nor for the geometry'

   dict['root']= version

## --- fluka
## -------------------------------------------------------------------------------------------
   print '      - identify FLUKA version: .',
   sys.stdout.flush()
   r0,e0= syscall('$FLUPRO/flutil/lfluka -o exe -m fluka')
   print '.',
   sys.stdout.flush()
   os.system('rm -f exe.map; echo STOP > ooo;')
   print '.',
   sys.stdout.flush()
   r1,e1= syscall('./exe < ooo')
   os.system('rm -f exe ooo')
   print '.',

   if e1:
          print ' ---> trouble compiling/linking against libflukahp.a, exit'
          print '      here the log:'
          print e0,' -- ',e1

   fort11= open('fort.11','r')
   all= fort11.readlines()
   fort11.close()

   for l in all:
      if l.find(r' Ferrari DATE: ')!=-1:
        version= l.strip().split(' ')[0].lower().replace('fluka','') +'.'+ l.strip().split(' ')[2].split('.')[0]

   if version in ['2006.3','2006.3b']:
     r2,e2= syscall('$FLUPRO/flutil/lfluka | grep ffortran')
     options= ''

     for l in r2.strip().split(' '):
        if l in ['g77','f77']: continue
        if l=='-v': break
        options+= ' '+l
        if l=='-Wall': options+= '\\n        '
     print version
   else:
         print version,'\n ---> trouble finding correct version of FLUKA'
         print '      only 2006.3 (or later) is supported at this time, exit'
         sys.exit(2)

   os.system('rm -f fort.11 flukahp.map')
   dict['fluka']= version
   dict['options']= options


   return dict



##
## --- 

if __name__ == "__main__":


  dict= determineStuff('skip')

##
## --- flukaGUI specifics for makefile
##     if no user routines were specified do real configuration/setup
##     otherwise just update the FLUKA user routine configuration in existing makefile
##
##     makefile - src/makefile - pgmdrv/makefile - patch/makefile - src/config.h
##

  if dict['user']:
    if not os.path.isfile(os.path.join('src','makefile')):
      print ' ---> trouble perform ',sys.argv[1],' with no arguments one time first, exit'
      sys.exit(2)

    print '     only modify makefile to link against (changed) FLUKA user routines'

##
## --- src/makefile

    try: mf= open(os.path.join('src','makefile'),'r')
    except: sys.exit(' ---> trouble with src/makefile.h, exit')
    makefile= mf.readlines()
    mf.close()

    aline= 'userRoutines='
    compile= '\n\t' + r'$(FC) $(FFLAGS) -c $^' + '\n\n'
    doWrite= True

    mf= open(os.path.join('src','makefile'),'w')
    for l in makefile:
       if l.startswith(aline):
         doWrite= False
         for file in dict['user'].strip().split(' '):
            if len(aline.split('\n')[-1])>110: aline+= '\\n               '
            aline+= ' '+file
         mf.write(aline+'\n\n')

       if l.startswith('## -x- '): doWrite= True

       if l.startswith('## -x- neither'):
         mf.write(l+'\n')
         doWrite= False
         for file in dict['fortran'].strip().split():
            bline= file.split('/')[-1].replace('.f','.o') + ': ' + file+compile
            mf.writelines(bline)

       if doWrite: mf.write(l)

    mf.close()

##
## --- config.h

    try: cf= open(os.path.join('src','config.h'),'r')
    except: sys.exit(' ---> trouble with src/config.h, exit')
    config= cf.readlines()
    cf.close()

    cf= open(os.path.join('src','config.h'),'w')
    for l in config:
       cf.write(l)
       if l.startswith('// -x-'): break

    for f in dict['fortran'].split(' '):
       n= dict['user'].replace(f.split('/')[-1].replace('.f','.o'),'')
       dict['user']= n

    all= dict['user'].strip()+' '+dict['fortran'].strip()
    list= [item.split('/')[-1].split('.')[0] for item in all.split()]

    cf.write('\n char *listOfUserRoutines= "' + all.strip() + ' ";\n')
    cf.write(' int nrOfUserRoutines=' + str(len(list)) + ';\n')

    dates= ' char *theirDates[24]= { '
    for i in list:
       if len(dates.split('\n')[-1])>110: dates+= '\n                    '
       dates+= '"' + dict['dates'][i] + '",'
    cf.write(dates.strip(',')+' };\n')

    cf.close()


## === setup ===
## ---
  else:

   replace= [ '## ---------------------------\n', 'export _libg2c= ' + dict['libg2c'] + '\n',
              'export _rootversion= '+ dict['root'].replace('/','.') +'\n', 'export _flukaversion= '+ dict['fluka'] +'\n',
              'export _flukaoptions= '+ dict['options'] +'\n','## ---------------------------\n\n','XLIB= '+ xlib +'\n',
              'prefix= '+ dict['prefix'] +'\n' ]

   config= [ '//\n// --- flukaGUI ---\n//\n\n',
             ' char *_here= "'+ pwd +'/src";\n',
             ' char *_prefix= "'+ dict['prefix'] +'";\n\n',
             ' char *flukaVersion= "' + dict['fluka'] + '";\n',
             ' char *rootVersion= "' + dict['root'] +  '";\n', '\n// -x- do not change this line\n\n',
             ' char *listOfUserRoutines;\n int nrOfUserRoutines=0;\n char *theirDates[24];\n' ]

   if xlib: config.insert(1,'#define FOUND_XLIB 1\n\n')

##
## --- src/makefile

   try: mf= open(os.path.join('src','Makefile.in'),'r')
   except: sys.exit(' ---> trouble with src/Makefile.in, exit')
   makefile= mf.readlines()
   mf.close()

   try: mf= open(os.path.join('src','makefile'),'w')
   except: sys.exit(' ---> trouble opening src/makefile for writing, exit')
   for l in makefile:
      if l.startswith('__replace__'): mf.writelines(replace)
      else: mf.write(l)
   mf.close()

##
## --- makefile

   try: mf= open('Makefile.in','r')
   except: sys.exit(' ---> trouble with Makefile.in, exit')
   makefile= mf.readlines()
   mf.close()

   try: mf= open('makefile','w')
   except: sys.exit(' ---> trouble opening src/makefile for writing, exit')
   for l in makefile:
      if l.startswith('__replace__'): mf.writelines(replace)
      else: mf.write(l)
   mf.close()

##
##-- pgmdrv/makefile
##   ---------------

   try: mf= open(os.path.join('pgmdrv','Makefile.in'),'r')
   except: sys.exit(' ---> trouble with pgmdrv/Makefile.in, exit')
   makefile= mf.readlines()
   mf.close()

   try: mf= open(os.path.join('pgmdrv',dict['fluka'],'makefile'),'w')
   except: sys.exit(' ---> trouble opening pgmdrv/version/makefile for writing, exit')
   for l in makefile:
      if l.startswith('__replace__'): mf.writelines(replace)
      else: mf.write(l)
   mf.close()

##
## --- src/config.h

   try: mf= open(os.path.join('src','config.h'),'w')
   except: sys.exit(' ---> trouble opening src/config.h for writing, exit')
   mf.writelines(config)
   mf.close()

##
## --- patch/makefile
   os.system('cp patch/makefile patch/root_v'+dict['root'].replace('/','.')+'/.')

   os.system('cd ./src; make proper')

   print '     --- flukaGUI package directory:\n                                     ' + pwd+'/src'

