
//
//    (feb'07 19-02:07) - version v.07a
//                                                                                 the FLEUR project
// -------------------------------------------------------------------------------------------------
//                                                                        author: anton.empl@cern.ch
// --- FLUKA user back-end project -Fbe-
//     attempt to extract a more flexible way to run pgmdrv() and leave the FLUKA side
//     of things as untouched as possible, based on new  pgmdrv.f (jun'04)
//
//      nov'04 29-00:54
//    - pickup the PLOTGEOM given scan parameters through a common/struct mapping and
//      hook a couple variables with CALLs into here
//    - taylored pgmpds to not read anymore
//


#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include <sys/types.h>
#include <fcntl.h>

#include <error.h>

#include <string.h>
#include <time.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/shm.h>

#include "scanGeometry.h"
#include "debug.h"


//
// --- fork/pipe communication ---

static int pid=-1,wdes[2],rdes[2];
struct { int mode,usrbin; } v0;
struct { float x,y,z; int rotID; } v1;

int nNstor; void *nstorPtr;
aptS* apt;
pgmaxS empty={0},p={0},*pgmaxs=&p;
aLS q,*aL=&q;
int *LPGMOP;

int trouble;
int FLUKA_OK=1;

// --- sep'06 09-19:13
//     all this to catch trouble on the child process side - more complicated
//     than I first thought - usually FLUKA said goodbye
// ---
static void sigchld_handler(int val) {
   int w,status;
   DEBUG(printf(" -o-> sigchld %i  - %i\n",pid,pgmaxs->voidType);)

   do { w= waitpid(-1,&status,WUNTRACED|WNOHANG);
        if (w>0 && pid>0 && w==pid) if (FLUKA_OK) { FLUKA_OK= 0; kill(getpid(),SIGUSR1); break; }
        if (w==-1) break;
        DEBUG(printf(" ---> w %i\n",w);)
        if (pid==0) _exit(0);
        // -- if (WIFEXITED(status)) { printf("exited, status=%d (%i)\n", WEXITSTATUS(status),pid); }
        // --  else if (WIFSIGNALED(status)) { printf("killed by signal %d\n", WTERMSIG(status)); }
        // --  else if (WIFSTOPPED(status)) { printf("stopped by signal %d\n", WSTOPSIG(status)); }
        // --  else if (WIFCONTINUED(status)) { printf("continued\n"); }
      }
        while (!WIFEXITED(status) && !WIFSIGNALED(status));
   }
// --- 
static void sigpipe_handler(int val) { DEBUG(printf(" -sigpipe- %i (%i)\n",getpid(),pid);)
                                       if (FLUKA_OK) { FLUKA_OK= 0; kill(getpid(),SIGUSR1); }}


void pgmdrv_(int*,char*,int*,char*);
void plotgm_(double*,char*);
void myrot_(int*);

char inpDate[32],geoDate[32],geoFile[256];
char *getInpDate() { return(inpDate); }
char *getGeoDate() { return(geoDate); }
char *getGeoFileName() { return(geoFile); }

//
// --- aug'06 07-13:21
//     make all simpler for FLUKA code, no more entry just a flag
int iflgeo=-1;
double what[6]; char sdum[9];
//


// --- from FLUKA ---
void cnstor_(int *n,void *arg) { nstorPtr= arg; nNstor= *n; }
void capr06_(void *arg) { apt= (aptS*)arg; }
void cpgmaxs_(void *arg) { pgmaxs= (pgmaxS*)arg; }
void cdraw_(float *c, short *m, int *n) { aL->c= c; aL->m= m; aL->np= n; aL->n= *(aL->np); }
void clpgmop_(int *ptr) { LPGMOP= ptr; }
void lextra_(int  *itype, double limits[]) { int i; pgmaxs->voidType= *itype; for (i=0;i<6;i++) pgmaxs->limits[i]= limits[i]; }

void pgmemo_();
void mygreg_(double*,double*,double*,int*);
void getmed_(int*,char*);
int  rotprbin_(int*);
// ---

static int nw=0;
static wormS *wp,*w=NULL;
static void *wptr,*shmptr;



//
// --- this one may not be needed but just in case
//     how about running through a FLUKA input file that references a geometry file
//     if the path is absolute then it will be wrong if the file ever is moved somewhere else
//     on the other hand just a relative path will break unleass we are in the place where both
//     files sit - so we need to temper with the relative path
//
    int tpwd(char *filename, char *extra)
// ---------------------------------------
{

DEBUG(printf(" ---> filename _%s_ \n",filename);)

char str[256],directoryName[256]="./";
int n= strlen(filename);
while (filename[n]!='/' && n-->0);

if (n>0) strncpy(directoryName,filename,n+1);
DEBUG(printf(" ---> input directoryName  _%s_   (%i)\n",directoryName,n);)

FILE *fp= fopen(filename,"r");
if (fp) {
          while (fgets(str,255,fp))
               if (!strncmp(str,"GEOBEGIN",8)) {
                 if (strlen(str)>39 && strncmp(&str[30],"          ",10))  {
                   DEBUG(printf(" _%s",str);)
                   DEBUG(printf(" ---> YES EXTRA FILE ASKED FOR\n");)
                   fgets(str,255,fp);
                   n= strlen(str)-1; while (str[n-1]==' ') n--; str[n]= '\0';
                   strcpy(extra,directoryName); strcat(extra,str);
                   if (str[0]!='/' && str[0]!='~') strcpy(filename,directoryName);
                     else strcpy(filename,str);

                   DEBUG(printf(" ---> %i _%s_\n",strlen(extra),extra);)
                   fclose(fp); return 1; }
                 fclose(fp); return 0; }
        }

return -1;
}


// --- here's were we start up FLUKA
//     get a possible extra geometry file correctly inlined, needs some monkey business
//     we initially used pgmdrv() but now the logic is too complicated to have a dedicated
//     FLUKA input file scanner, hence we actually tailor flukam() slightly for 2005.6
//     and initial 2006.1 versions of FLUKA - so scan input and prepare stuff for 'plotgeom'
//     so that later we can perform individual scans
//
    pgmaxS *attach2geometry(char *filename)
// -----------------------------------------
{

char geometry[81],extra[256];
int i,n, ng=0,nf=strlen(filename);

strcpy(extra,"");
strcpy(geometry,filename);
int status= tpwd(geometry,extra);

strcpy(geoFile,"");
strcpy(geoDate,"");
if (status>=0) { DEBUG(printf(" ---> FLUKA input file seems alright\n");)
                 ng= strlen(geometry);

   struct stat myStat;
   if (status>0) { DEBUG(printf("    > included geometry file _%s_ %i %i\n",geometry,nf,ng);)
                   stat(geometry,&myStat);
                   strcpy(geoDate,ctime(&myStat.st_mtime));
                   n= strlen(geoDate)-1; while (n>0 && geoDate[n--]==' '); geoDate[n+1]= '\0';
                   strcpy(geoFile,extra); }

   stat(filename,&myStat);
   strcpy(inpDate,ctime(&myStat.st_mtime));
   n= strlen(inpDate)-1; while (n>0 && inpDate[n--]==' '); inpDate[n+1]= '\0';

//
// -----=== fork ===-----
// -----
//
    signal(SIGCHLD,sigchld_handler);
    signal(SIGPIPE,sigpipe_handler);

    pipe(wdes);
    pipe(rdes);

    static key_t key= 0x00001235;
    int shmid= shmget(key,1000000,(IPC_CREAT|0660));
    shmptr= shmat(shmid,0,0);
    shmctl(shmid,IPC_RMID,0);

    trouble= 0;

    if ((pid=fork())==0) {       // ------------------------------------------------------ child / fluka -----------
      close(wdes[1]);
      close(rdes[0]);
//
//       may'06 11-13:23
// ----- actual call into FLUKA, read through input file here
//       we push stderr into a file - indicate to parent our problem in case flabrt()
//       and other parse problems with pgmaxs->voidType
//

      int fd0= open("stderr",O_CREAT|O_WRONLY,00664); close(2); dup(fd0); close(fd0);
//   ==========================================

      pgmdrv_(&nf,filename,&ng,geometry);
      pgmemo_();

      DEBUG(printf(" ===> hey after pgmdrv_ voidType= (%i)\n",pgmaxs->voidType);)

      if (pgmaxs->voidType<1) {
        fprintf(stderr," FLUKA subprocess just quit\n");
        fprintf(stderr,"  does this input file work with the corresponding version of FLUKA?\n");
        _exit(0); }

// ---
      n=0; while (n<60 && pgmaxs->title[n++]==' '); n--; for (i=n;i<60;i++) pgmaxs->title[i-n]= pgmaxs->title[i];
      n= 59-n; while (n>0 && pgmaxs->title[n--]==' '); pgmaxs->title[n+2]= '\0';
      n=0; while (n<80 && pgmaxs->runtit[n++]==' '); n--; for (i=n;i<80;i++) pgmaxs->runtit[i-n]= pgmaxs->runtit[i];
      n= 79-n; while (n>0 && pgmaxs->runtit[n--]==' '); pgmaxs->runtit[n+2]= '\0';

//   ==========================================
      int fd1= open("/dev/tty",O_WRONLY); close(2); dup(fd1);

      write(rdes[1],pgmaxs,sizeof(pgmaxS));
//
// -----
//
      static char buf[4];
      do { read(wdes[0],buf,1);

           if (buf[0]=='s') {
             read(wdes[0],&v0,sizeof(v0));
             int binID= v0.usrbin;
             *LPGMOP= v0.mode;
             DEBUG(printf(" ---child--- call plotgm()  %i %i\n",*LPGMOP,binID);)

             nw= 0; wptr= shmptr;

             read(wdes[0],pgmaxs,sizeof(pgmaxS));
//          ==================================
             myrot_(&binID);
             plotgm_(what,sdum);
//          ==================================
             write(rdes[1],&nw,4);
             write(rdes[1],w,nw*sizeof(wormS));
             }

           if (buf[0]=='p') {
             read(wdes[0],&v1,sizeof(v1));
             DEBUG(printf(" ---child--- call aPoint()  %f %f %f - %i \n",v1.x,v1.y,v1.z,v1.rotID);)
             double x=v1.x, y=v1.y, z=v1.z; int rotID=v1.rotID;
             mygreg_(&x,&y,&z,&rotID);
             DEBUG(printf(" ---child--- %i %i _%s_ _%s_\n",apt->nreg,apt->nmat,apt->rname,apt->mname);)
             write(rdes[1],apt,sizeof(aptS));
             }

           if (buf[0]=='l') {
             aL->n= *(aL->np);
             DEBUG(printf(" ---child--- n %4i\n",aL->n);)
             write(rdes[1],&aL->n,sizeof(int));
             write(rdes[1],aL->c,aL->n*sizeof(float));
             write(rdes[1],aL->m,aL->n*sizeof(short));
             }

           if (buf[0]=='m') {
             int nmate;
             char a[9];
             read(wdes[0],&nmate,sizeof(int));
             getmed_(&nmate,a);
             DEBUG(printf(" ---child--- m %4i _%s_\n",nmate,a);)
             write(rdes[1],a,sizeof(a));
             }

           if (buf[0]=='r') {
             int binID;
             read(wdes[0],&binID,sizeof(int));
             int rotID= rotprbin_(&binID);
             DEBUG(printf(" ---child--- r %4i _%4i_\n",binID,rotID);)
             write(rdes[1],&rotID,sizeof(int));
             }

         } while (buf[0]!='x');

      DEBUG(printf(" ---child--- _exit() --- ---\n");)
      _exit(0);
//
// -----
//
  }

  else {     // -------------------------------------------------------------------------- parent ------------------
         close(wdes[0]);
         close(rdes[1]);

         pgmaxs->voidType= 0;
         read(rdes[0],pgmaxs,sizeof(pgmaxS));

         DEBUG(printf(" ---parent--- read/parent: _%s_ %i %f %f\n", pgmaxs->title,pgmaxs->voidType,pgmaxs->X0,pgmaxs->X1);)
         if (pgmaxs->voidType<1) pid= -1;
  }
//
// -----
// -----=== fork ===-----
//
 }
  else return NULL;

return pgmaxs;
}



//
// --- apr'06 25-12:53
//     this one to close out our forked FLUKA child
//
    void shutFlukaDown()
// ----------------------
{

if (pid<0) return;
DEBUG(printf(" ---fork--- write/parent --- close FLUKA child down\n");)
write(wdes[1],"x",1);
DEBUG(printf(" ---> after wait() before shmdt()\n");)
shmdt(shmptr);
pid= -1;
}




//
// --- actively call plotgm() this way we can implement setting up the scan configuration
//     also will permit reentering without reading input file again
//     single_plotgm() is a fortran entry point into init_plotgm() - the main part is called
//     from init_pgmdrv() with the START/PLOTGEOM option parameter
//
// --- aug'06 07-13:21
//     changed it all such that we have minimal impact on FLUKA code
//     init_plotgm() turns into simple plotgm() with a flag to skip top
//     see also pgmpdf() which perform partial stuff according to our flag IFLGEO
//
// --- apr'06 25-11:12
//     only trigger call to single_plotgm() from here since it resides in our child process
//     this places also the 'write call-backs' below into the child's namespace/memory
//     so that we will just sit here and block on reading back the information - the required
//     pipes were opened above
//
// --- sep'06 18-12:23
//     implemented checks on status of FLUKA child
//     jump back to GUI if we received a sigpipe earlier somewhere
//     holds for all the hooks into our child here
//
    wormS *scan1time(int *Inw, int RM, int id)
// --------------------------------------------
{
nw= 0;
v0.mode= RM; v0.usrbin= id;

DEBUG(printf(" ---parent--- (scan1time) write/parent (s)\n");)
write(wdes[1],"s",1); if (!FLUKA_OK) return NULL;
write(wdes[1],&v0,sizeof(v0));

write(wdes[1],pgmaxs,sizeof(pgmaxS));

read(rdes[0],&nw,4);
w= (wormS*)malloc(nw*sizeof(wormS));
read(rdes[0],w,nw*sizeof(wormS));

*Inw= nw;
return(w);
}



//
// --- wrapper to call wrapped georeg_() of FLUKA
//     we trigger the actual action in our FLUKA child process by means of 'p'
//     apr'06 28-11:49
//
    aptS *aPoint(float x, float y, float z, int rotID)
// ---------------------------------------------------
{
static aptS ret;

DEBUG(printf(" ---parent--- (aPoint) write/parent (p)\n");)
write(wdes[1],"p",1); if (!FLUKA_OK) return NULL;

v1.x= x; v1.y= y; v1.z= z; v1.rotID= rotID;
write(wdes[1],&v1,sizeof(v1));
read(rdes[0],&ret,sizeof(aptS));

if (ret.nmat==9999 || ret.nmat==999999) ret.nmat= 1;
return &ret;
}



//
// --- hey, get a material name from number
//     jul'06 06-13:43
//
    void getMaterialName(int *i, char *a)
// ---------------------------------------
{
write(wdes[1],"m",1); if (!FLUKA_OK) return;
write(wdes[1],i,4);

char b[9]; read(rdes[0],b,sizeof(b)); strcpy(a,b);

}



//
// --- ask FLUKA if we have a roprbin defined for this binning
//     sep'06 18-22:23
//
    int rotprbin(int *binID)
// --------------------------
{
write(wdes[1],"r",1); if (!FLUKA_OK) return -1;
write(wdes[1],binID,4);

int rotID; read(rdes[0],&rotID,4);
return rotID;
}



//
// --- may'06 08-15:27
//     our own approach to get a scanning of the 2D FLUKA geometry slice
//
    aLS *getLines()
// -----------------
{
write(wdes[1],"l",1); if (!FLUKA_OK) return NULL;

read(rdes[0],&aL->n,sizeof(int));
if (aL->c) free(aL->c); aL->c= (float*)malloc(aL->n*sizeof(float));
float *f= aL->c; read(rdes[0],f,sizeof(float)*aL->n);
if (aL->m) free(aL->m); aL->m= (short*)malloc(aL->n*sizeof(short));
read(rdes[0],aL->m,sizeof(short)*aL->n);

DEBUG(printf(" ---parent--- (lines) write/parent (l)\n");)
return aL;
}




//
// --- these are called from the FLUKA side bringing over our worms
//     just replaced the fortran write with these special C calls -- pgmufz()
//
//     apr'06 27 15-20
//     slight adaptation in context of fork() -  now our w[] holds ONLY the pointers to
//     the x/y points of each worm - they are actually mapped into a 1MByte shared memory
//     region to communicate them back to parent process
// --------------------------------------------------------------------------------------------------------------------------------
void write_1_(int *n, int *l) { if (*n) { w= (wormS*)realloc(w,(nw+*n)*sizeof(wormS)); wp= &w[nw]; wp->NFRZR= *n; wp->LFRZR= *l; }}
void write_2_(int *n, int *i, int *l) { if (*n) { wp= &w[nw]; wp->NWORM= *n; wp->IHEAD= *i; wp->n= wp->LENSHR= *l; }}
void write_3_(float *x, float *y)
{
   if (wp->n>0) {
     int i; float *p; wp= &w[nw];
     p= (float*)wptr; for (i=0;i<wp->n;i++) p[i]= x[i]; wp->x= p; wptr+= wp->n*sizeof(float);
     p= (float*)wptr; for (i=0;i<wp->n;i++) p[i]= y[i]; wp->y= p; wptr+= wp->n*sizeof(float);
     //-ae- DEBUG(printf(" ---> wptr %i\n",(wptr-shmptr));)
     nw++; }
}
// --------------------------------------------------------------------------------------------------------------------------------

